#!/usr/bin/perl -w
#
# This script is available under the Gnu General Public Licence (GPL)
#
# Written by James Bromberger (james@rcpt.to).
#
#
use strict;
use lib qw(../lib);
use Email;
use CGI;
use DBI;
use Person;
use Form;

my $q = new CGI;
my $dbh = DBI->connect("dbi:mysql:database=LCA2003-Reg") or die "Can't connect";

my @people = Person->find(-dbh => $dbh);

sub email_person {
	my $person = shift;
	my $owing = shift;

	my $time_delta_days = int((time() - $person->date_entered) / (3600 * 24)) + 1;

	if ($time_delta_days > 56) {
		warn sprintf "Giving up on %s (%s) who registered on %s (time delta %s)", 
			$person->name, $person->email_address,
			scalar localtime($person->date_entered),
			$time_delta_days;
		return;
	}

	if ($time_delta_days % 14 != 0) {
		#warn sprintf "Skipping emailing %s (%s) because they registered on a %s, and today is %s, delta is $time_delta_days", $person->name, $person->email_address, scalar(localtime($person->date_entered)), scalar(localtime);
		return;
	}
	my $date_registered = scalar localtime $person->date_entered;
	my $custnum = $person->custnum;
	my $text = <<FOO;

Hello from Linux.conf.au!

We have your details registered on the Linux.conf.au web site that you want to 
attend the conference in January 2003, but so far we haven't recieved full 
payment for your registration. Your outstanding debt is AU\$$owing. 

You supposedly registered at $date_registered +0800.

If you wanted to pay by credit card, you can do this now from the checkout by 
visiting http://www.linux.conf.au/register/checkout.cgi and using your 
customer number '$custnum'. You'll get a receipt by email within (roughly) 
30 minutes of processing.

If you were sending a cheque or other forms of payment, then this probably 
means we just haven't received it yet, or haven't got around to entering this 
transaction into our database, in which case you can ignore this email. 
When we get your payment, and have cleared it, we'll issue a receipt by email.
We're going to remind you weekly for three weeks or until we receive and 
process your payment, after which we'll stop bugging you! At any time you 
can get your details we have for you emailed to you by visiting the 
registration page at http://www.linux.conf.au/register/.

If you don't know anything about this, then someone has probably put your 
email address in and tried to subscribe you; please email the organisers of 
the conference at lca-organisers\@linux.conf.au and we'll sort it out.

Regards,

  LCA Registration Scripts
  for Linux.conf.au 2003
  for Linux Australia, Inc.
-- 
 Linux.conf.au: The Australian Linux Techinical Conference
 The University of Western Australia, Perth, January 22 to January 25 2003
 Web: http://www.linux.conf.au/    IRC Help: #lca2003 on irc.openprojects.net

 Linux (R) is a registered trademark of Linus Torvalds
 Linux Australia (TM) is a trademark of Linux Australia Inc. ABN 56 987 117 479
FOO

	warn "Sending email to " . $person->email_address;
	#warn "time_delta_days = $time_delta_days for $date_registered for person " . $person->name ;
	Email->new(-to => $person->email_address, -subject => '[LCA2003] Unpaid registration for Linux.conf.au 2003', -content => $text);
	return;
}


my $summary_text;
my $registered = 0;
foreach my $person (@people) {
	my $amount_total = $person->amount_total;
	my $amount_paid = $person->amount_paid;
	my $amount_owing = $person->amount_owing;
	$summary_text.= sprintf "%s (%s) owes %s, paid %s, ", $person->name, $person->email_address, $amount_total, $amount_paid;
	if (($amount_paid == 0) && ($amount_owing >0)) {
		# This person hasn't paid anything
		$summary_text.= "NO PAYMENTS\n";
		email_person($person, $amount_total - $amount_paid);
	} elsif ($amount_paid < $amount_total) {
		# This person hasnt paid it all
		$summary_text.= "SOME PAYMENTS\n";
		email_person($person, $amount_total - $amount_paid);
	} elsif ($amount_paid > $amount_total) {
		# This person has overpaid
		$summary_text.= "OVER PAID\n";
		$registered++;
	} else {
		#This is OK!
		$summary_text.= "OK\n";
		$registered++;
	}
}

#warn "Updating registered to $registered";
Email->new(-to => 'james@rcpt.to', -subject => "[LCA2003] Daily Registrant Status Summary", -content => $summary_text) if defined $summary_text;
my $sql = "UPDATE Variable set Value = ? where Name = 'Num_Registered_And_Paid'";
my $sth = $dbh->prepare($sql);
my $res = $sth->execute($registered);
