#!/usr/bin/perl -w
#
# This script is available under the Gnu General Public Licence (GPL)
#
# Written by James Bromberger (james@rcpt.to).
#
#
use lib qw(lib);
use strict;
use CGI;
use DBI;
use Person;
use Form;
use Variable;

my $q = new CGI;
my $dbh = DBI->connect("dbi:mysql:database=LCA2003-Reg") or die "Can't connect";
my $f = Form->new(-query => $q, -dbh => $dbh);

sub html_head {
	print $q->header;
	print $f->html_header();
	#print $q->start_html;
	print $q->h2('Checkout');
}
sub html_tail {
	print $q->end_html;
	print $f->html_footer();
}

sub person_not_found {
	html_head();
	print $q->p('Sorry, we dont know who you are.');
	html_tail();
}

sub person_ask_custnum {
	html_head();
	print <<FOO;

<p>
Your payments can be done in several ways. We prefer Credit Cards, since 
they can be processed now, with little overhead on the volunteers who are 
running Linux.conf.au. We can handle Australian personal cheques, and 
international bank drafts.
</p>

<p>
If you don't know how much you owe, then check out 
<a href='my_details.cgi'>your details</a>. If you are not going to pay by 
credit card, then dont continue. Contact us and tell us how you are going to 
pay.
</p>

<p>We can only accept <strong>Visa, Mastercard, Bankcard, and American Express (AMEX)</strong>.</p>

FOO
	printf "<form method='post' action='%s'>Your customer number: <input name='c'><input type='submit' value='Go'></form>", $q->url;
	html_tail();
}

sub person_confirm {
	my $person = shift;
	html_head();
	if ($person->amount_owing <= 0) {
		printf "<p>Hello %s. You don't owe us anything.</p>", $person->name;
	
	} else {
		printf "<p>Hello %s. You owe Linux.conf.au AU\$%s for your registration you have filled out.</p>", $person->name, sprintf("%.2f", $person->amount_owing );

		print <<FOO;

<p>
There are several steps to paying for this by credit card. At every stage, 
you will be using encrypted web traffic (HTTPS). You will be sent to 
our credit card processing broker, CommSecure. Your confidential credit 
card details are never given to LCA. The steps are:
</p>

<ol>
<li>Confirm this payment by submitting this form, which will take you 
to CommSecure.
<li>Fill in your credit card details there. 
<li>After your payment is processed, you will be returned here.
</ol>

<p>You will be emailed a receipt from Linux.conf.au upon us verifying the transaction was successful.
</p>

<p>We can only accept <strong>Visa, Mastercard, Bankcard, and American Express (AMEX)</strong>.</p>

FOO
	
		commsecure_form($person);
	}
	html_tail();
}

sub commsecure_form {
	my $person = shift;
	my $url = Variable->new(-dbh => $dbh, -name => "CommSecure_URL");
	if (not defined $url) {
		warn "CommSecure_URL not defined!";
	}
	my $merchant_num = Variable->new(-dbh=>$dbh, -name => "CommSecure_MerchantNum");
	use CommSecure::Request;
	# NB, we multiple the amount here by 100 because commsecure works in cents
	my $request = CommSecure::Request->new(-dbh => $dbh, -description => 'Linux.conf.au Registration', -host => $person->host, -amount => $person->amount_owing * 100, -merchantnum => $merchant_num, -custnum => $person->custnum, '-gen-form' => 1);
	print $request;
	return 1;
}

if (defined $q->param('c')) {
	#warn "Looking for person matching custID " . $q->param('c');
	my $person = Person->new(-custnum => $q->param('c'), -dbh => $dbh);
	if (not $person) {
		person_not_found();
		#$dbh->disconnect;
		exit;
	}
	person_confirm($person);

} else {
	person_ask_custnum();
}

