package CommSecure::Crypto;

use strict;
use Math::BigInt;
use Digest::MD5;
use MIME::Base64;

my $VERSION = "0.9.1";
sub Version { $VERSION; }

sub inverse_int {
	my ($e, $modulo) = @_;
	#find the inverse of e mod modulo
	

	# do some euclid's stuff

	#printf STDERR "    * Finding magic numbers...\n";

	my $x1 = new Math::BigInt $modulo;
	my $x2 = new Math::BigInt $e;
	my @magic = ();

	while ($x2 != "0") {
#		my ($div, $rem) = $x1->bdiv($x2);
		my $div = $x1/$x2;
#		my $rem = $x1%$x2;
		my $rem = $x1-($x2*$div);
		# at this point, $x1 = $div * $x2 + $rem should always hold
		push @magic, $div;
	#	print "D: ", $div, "\nR:", $rem, "\n";
		$x1=$x2;
		$x2=$rem;
	}
	
	#printf STDERR "    * Working the magic...\n";

	my $a = new Math::BigInt "0";
	my $b = new Math::BigInt "1";
	my $c = new Math::BigInt "1";
	my $d = new Math::BigInt "0";
	my $tmp1 = new Math::BigInt "0";
	my $tmp2 = new Math::BigInt "0";

	foreach my $x (@magic) {
		$tmp1 = ($x*$b)+$a;
		$tmp2 = ($x*$d)+$c;
		$a = $b;
		$b = $tmp1;
		$c = $d;
		$d = $tmp2;
	}

	if (($#magic%2)==1) {
		# the inverse is -1 * $a in mod (p-1)(q-1)
#		printf STDERR "    * Need to watch out on this one... More black magic\n";
		$a *= "-1";
		$a += $modulo;
		$a %= $modulo;
		
	}

	die "Inverse: magic table didn't compute - tmp1 was ",$tmp1 unless ($tmp1==$modulo);
	die "Inverse: magic table didn't compute - tmp2 was ",$tmp2 unless ($tmp2==$e);

	# final check
	die "Inverse: Doesn't compute" unless ($a*$e)%$modulo=="1";

	return $a;
}

sub bexp_int {
	my ($base, $expon, $modulus) = @_;
	
	### Convert the exponent to binary ###
	my $quot = $expon + 0;	# copy exponent
	my $m1   = Math::BigInt->new(1);
	my @d_exp=();
	while ($quot > 0) {
		if ($quot % 2) { #odd
			$quot--;
			unshift @d_exp, 1;
		} else { #even
			unshift @d_exp, 0;
		}
		$quot /= 2;
	}	
	#print " ", join('',@d_exp);
	foreach (@d_exp) {
		$m1 *= $m1;	#square $m1
		#$m1 %= $modulus if ($m1 >= $modulus);	#Reduce mod n
		$m1 *= $base if ($_);
		$m1 %= $modulus; #if ($m1 >= $modulus);	#Reduce mod n
		
		#$m1 = $modulus - $m1 if ($modulus - $m1 < $m1);   #use smaller numbers mod n
	}
	return $m1;
}

######################################################################
#### libgmp functions called through gmpfunc
######################################################################

# calls the bigexp program to evaluate it with libgmp.
# is there a better way?

sub gmpfunc {
	my $gmpfuncprog = "/usr/local/bin/gmpfunc";
	my $subfunc = shift;
	my $pipedata = "";
	$pipedata .= sprintf("%s\n", $_) foreach @_;

	$pipedata =~ s/^[\+-]//gm; # strip off leading +'s

	my $ans = `echo "$pipedata"|$gmpfuncprog $subfunc`;
	chomp $ans;
	my $retval = new Math::BigInt $ans;
	return $retval;
}

sub bexp {
	my ($base, $exp, $mod) = @_;
	return gmpfunc ("powm", $base, $exp, $mod);
}

sub inverse {
	my ($n, $mod) = @_;
	return gmpfunc ("invert", $n, $mod);
}

######################################################################

sub bexp_pq {
	my ($base, $exp, $p, $q, $invpinmodq) = @_;
	#evaluate x^n in mod pq
	my $a = bexp($base, $exp, $p);
	my $b = bexp($base, $exp, $q);
	my $x = (($b - $a) * $invpinmodq);
	$x -= (($x/$q)-1)*$q;
	return (($p * $x) + $a) % ($p*$q);
}

sub hash_modulus {
	my ($data, $N) = @_;
	# if the modulus is less than 2^128, then we need to take the modulus
	# (should only be < 128 bits for testing purposes only)
	my $datahash = Digest::MD5::md5($data);

	# if it's sposed to be in the reverse order, tack in a reverse here:
	my @bytes = unpack("C*", $datahash);

	# translate MD5 hash into a big number
	my $numdata = new Math::BigInt 0;
	foreach (@bytes) {
		$numdata <<= 8; 
		$numdata |= Math::BigInt->new($_);
	}

	#modulus it
	$numdata %= $N;

	# just to check
	#warn "Decrypted is ", bexp($numdata, $d, $N);

	# translate this BigInt back into base256
	@bytes = ();
	while ($numdata!="0") {
		unshift @bytes, int($numdata&"255");
		$numdata >>= 8;
	}
	my $packed = pack "C*", @bytes;

	return $packed;
}


sub sign {
	my ($data, %key) = @_;
	my $datahash = Digest::MD5::md5($data);
	my $N = $key{"product"};
	my $e = $key{"e"};

	# if it's sposed to be in the reverse order, tack in a reverse here:
	my @bytes = unpack("C*", $datahash);

	# translate MD5 hash into a big number
	my $numdata = new Math::BigInt 0;
	foreach (@bytes) {
		$numdata *= 256; 
		$numdata += Math::BigInt->new($_);
	}

	#encrypt it
	$numdata = bexp_pq($numdata, $key{"e"}, $key{"product"}, $key{"p"},
				$key{"q"}, $key{"inv"});

	# just to check
	#warn "Decrypted is ", bexp($numdata, $d, $N);

	# translate this BigInt back into base256
	@bytes = ();
	while ($numdata!="0") {
		unshift @bytes, int($numdata&"255");
		$numdata >>= 8;
	}
	my $packed = pack "C*", @bytes;

	# and base64 encode it
	my $encoded = MIME::Base64::encode_base64($packed);

	$encoded =~ s/\n//g;
	return $encoded;
}

sub verify {
	my ($data, %key) = @_;
	my $N = $key{"product"};
	my $d = $key{"d"};
	# rip out the base64ness
	warn "data is $data";
	warn "N is $N";
	warn "d is $d";
	$data =~ s/[\n\r]//g;
	my $decoded  = MIME::Base64::decode_base64($data);

	# translate the monster into a BigInt
	my @bytes = (unpack "C*", $decoded);
	my $numdata = new Math::BigInt "+0";
	my $i = new Math::BigInt "+1";
	foreach (@bytes) {
		#$numdata <<= 8;
		$numdata *= 256;
		#$numdata |= $_;
#		warn "byte is $_";
		$numdata += $_;
		#$numdata += ($_*$i);
		#$i *= 256;
	}

	# decrypt it
	$numdata = bexp($numdata, $d, $N);

	# now we need to put it back into base256
	@bytes = ();
	while ($numdata!="0") {
		unshift @bytes, int($numdata&"255");
		$numdata >>= 8;
	}
	my $packed = pack "C*", @bytes;

	# and bingo
	return encode_base64($packed);
}

sub gen_key {
	my ($pstr, $qstr, $estr) = @_;

#printf STDERR <<cut
#Reads in p, q, e from stdin. (They'd better be primes)
#Well technically, e must only be relatively prime to (p-1)x(q-1)
#cut

	chomp $pstr;
	chomp $qstr;
	chomp $estr;

	my $p = new Math::BigInt $pstr;
	my $q = new Math::BigInt $qstr;
	my $e = new Math::BigInt $estr;

	my $product = new Math::BigInt $p;
	$product = $p*$q;

	my $modulo = ($p-"1")*($q-"1");
	my $d = inverse($e, $modulo);
	my $invpmodq = inverse($p, $q);

#printf STDERR "    * Found decryption key! And it's all good! HURRAH! =)\n\n";

#$output = (sprintf "%s\n%s\n%s\n", $product, $e, $d);

	# we want $d to be smaller cause the client has to decrypt it!
	if ($d>$e) {
	  my $tmp = $d;
	  $d = $e;
	  $e = $tmp;
	}
	
	my %key = ();
	$key{"product"} = $product;
	$key{"d"} = $d;
	$key{"e"} = $e;
	$key{"p"} = $p;
	$key{"q"} = $q;
	$key{"inv"} = $invpmodq;

	return %key;

#printf $output;
}

sub write_public_keyfile {
	my (%key) = @_;
	return (sprintf "Crypto.pm Keyfile v1.0\n%s\n%s\n0\n0\n0\n0\n",
		$key{"product"},
		$key{"d"});
}

sub write_private_keyfile {
	my (%key) = @_;
	my $product = $key{"product"}; chomp $product;
	my $d = $key{"d"}; chomp $d; 
	my $e = $key{"e"}; chomp $e; 
	my $p = $key{"p"}; chomp $p; 
	my $q = $key{"q"}; chomp $q; 
	my $inv = $key{"inv"}; chomp $inv;
	return (sprintf "Crypto.pm Keyfile v1.0\n%s\n%s\n%s\n%s\n%s\n%s\n",
		$product, $d, $e, $p, $q, $inv);
}

sub read_keyfile {
	open KEYFILE, shift or die "Can't open keyfile $_";
	die "Invalid key file" if (<KEYFILE> ne "Crypto.pm Keyfile v1.0\n");
	my %key = ();
	my $in;
	$in = <KEYFILE>; chomp $in; $key{"product"} = new Math::BigInt $in;
	$in = <KEYFILE>; chomp $in; $key{"d"} = new Math::BigInt $in;
	$in = <KEYFILE>; chomp $in; $key{"e"} = new Math::BigInt $in;
	$in = <KEYFILE>; chomp $in; $key{"p"} = new Math::BigInt $in;
	$in = <KEYFILE>; chomp $in; $key{"q"} = new Math::BigInt $in;
	$in = <KEYFILE>; chomp $in; $key{"inv"} = new Math::BigInt $in;
	return %key;
}

sub match_md5 {
    my ($a, $b) = @_;
    return 0 if (length $a ne length $b);
    my @aparts = unpack "C*", $a;
    my @bparts = unpack "C*", $b;
    foreach (@aparts) {
        return 0 if ($_ ne shift @bparts);
    }
    return 1;
}


1;

=head1 NAME

Crypto - A cryptography library that uses public key encryption

=head1 DESCRIPTION

This module provides a few functions that implement public key
encryption. Numbers are dealt with through Math::BigInt in all
cases. It also utilises the libgmp (GNU Multi-precision) library
through a helper application gmpfunc.

=head1 METHODS

=head2 Crypto::sign($data, $N, $e)
		
This function takes $data and signs it using the key $e and modulo $N.
It returns a Base64 encoded signature of the MD5 has of $data. It
returns a string of the same format accepted by Crypto::verify

=head2 Crypto::verify($data, $N, $d)

This function takes $data and verifies it was signed by the owner of
the key corresponding to $d and modulo $N.  $data is the Base64
encoded signature.  The return value should be compared with the hash
of the original (preferably using match_md5).

=head2 Crypto::match_md5($a, $b)

This function takes two MD5 hashes, and compares them byte by byte.
Why eq, cmp or == doesn't do this? *shrug*

Returns a non-zero value if the two hashes are the same.
Zero otherwise.

=head2 Crypto::gen_keys($pstr, $qstr, $estr)

This function generates a public and private key pair. Oddly, it takes
its arguments in string form. $pstr and $qstr are (preferably *large*)
primes. The function returns ($modulus, $e, $d) where $e and $d are 
the encryption and decryption keys (interchangeable). Returns Math::BigInt's

=head1 INTERNAL FUNCTIONS

Crypto also contains lots of functions it uses internally, though may
be useful for general cryptographic work.

=over 4

=item *

hash_modulus ($data, $N) takes some data, and produces the MD5 hash 
modulo $N. Use this for reducing the MD5 hash to less than 128-bits
for testing these functions with a key less than 128-bits. $N is a 
Math::BigInt.

=item *

inverse ($n, $mod) finds the inverse of $n in mod $mod.

=item *

bexp ($base, $exp, $mod) raises $base to $exp under $mod.

=back

=head1 LICENCE

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

=head1 AUTHOR

Copyright (C) 2002 Bernard Blackham <b_at_blackham.com.au>


=cut
