package CommSecure::Report;
use strict;

use Text::CSV_XS;

my $www_program = "/usr/bin/lynx";
my $www_args  = "-dump -source -post_data ";
my $commsecure_url = "https://clearance.commsecure.com.au/cgi-bin/Report.cgi/fromMain";

my $VERSION = "0.1";
sub Version { $VERSION; }

sub new {
	my $proto = shift;
	my $class = ref($proto)||$proto;
	my $self = {};
	bless $self, $class;
	my %args = @_;
	
	#defaults 9Sep02 - 9Sep03 - should cover everything.
	$self->StartYear("2002");
	$self->StartMonth("09");
	$self->StartDay("01");
	$self->StartHour("00");
	$self->StartMins("00");
	$self->EndYear("2003");
	$self->EndMonth("09");
	$self->EndDay("01");
	$self->EndHour("00");
	$self->EndMins("00");
	
	return if ((defined $args{'-Username'}) && !($self->Username($args{'-Username'})));
	return if ((defined $args{'-Password'}) && !($self->Password($args{'-Password'})));
	return if ((defined $args{'-StartYear'}) && !($self->StartYear($args{'-StartYear'})));
	return if ((defined $args{'-StartMonth'}) && !($self->StartMonth($args{'-StartMonth'})));
	return if ((defined $args{'-StartDay'}) && !($self->StartDay($args{'-StartDay'})));
	return if ((defined $args{'-StartHour'}) && !($self->StartHour($args{'-StartHour'})));
	return if ((defined $args{'-StartMins'}) && !($self->StartMins($args{'-StartMins'})));
	return if ((defined $args{'-EndYear'}) && !($self->EndYear($args{'-EndYear'})));
	return if ((defined $args{'-EndMonth'}) && !($self->EndMonth($args{'-EndMonth'})));
	return if ((defined $args{'-EndDay'}) && !($self->EndDay($args{'-EndDay'})));
	return if ((defined $args{'-EndHour'}) && !($self->EndHour($args{'-EndHour'})));
	return if ((defined $args{'-EndMins'}) && !($self->EndMins($args{'-EndMins'})));

	# now go fetch, if we have all the data.
	if ($self->Username && $self->Password) {
		return unless $self->fetch_data();
	}

	return $self;
}

sub fetch_data {
	my $self = shift;
	use IPC::Open2;
	my $pid = IPC::Open2::open2(\*RF, \*WF, "$www_program $www_args $commsecure_url") || die "Couldnt spawn";
	# write each param to WF and then close it for an eof
	print WF "User=".$self->Username."&\n";
	print WF "Password=".$self->Password."&\n";
	print WF "page=1&\n";
	print WF "displayFormat=CSV&\n";
	print WF "ReportType=TransReport&\n";
	print WF "StartYear=".$self->StartYear."&\n";
	print WF "StartMonth=".$self->StartMonth."&\n";
	print WF "StartDay=".$self->StartDay."&\n";
	print WF "StartHour=".$self->StartHour."&\n";
	print WF "StartMins=".$self->StartMins."&\n";
	print WF "EndYear=".$self->EndYear."&\n";
	print WF "EndMonth=".$self->EndMonth."&\n";
	print WF "EndDay=".$self->EndDay."&\n";
	print WF "EndHour=".$self->EndHour."&\n";
	print WF "EndMins=".$self->EndMins."&\n";
	print WF "---\n";
	close WF;
	# and now read our input
	my @records = <RF>;

	# remove trailing new lines and blank lines.
	chomp @records;
	for (my $i = 0; $i <= $#records; $i++) {
		delete $records[$i] if ($records[$i] eq "");
	}


	$self->rawresult(@records);

	# kill our children with less mess.
	waitpid $pid, 0;
	warn "Non-zero exit code from $www_program ($?)" if $?;

	return 1;
}

sub Username {
	my $self = shift;
	return $self->{'Username'} unless (@_);
	my $newvalue = shift;
	$self->{'Username'} = $newvalue;
}

sub Password {
	my $self = shift;
	return $self->{'Password'} unless (@_);
	my $newvalue = shift;
	$self->{'Password'} = $newvalue;
}

sub StartYear {
	my $self = shift;
	return $self->{'StartYear'} unless (@_);
	my $newvalue = shift;
	return unless $newvalue =~ /\d+/;
	$self->{'StartYear'} = $newvalue;
}

sub StartMonth {
	my $self = shift;
	return $self->{'StartMonth'} unless (@_);
	my $newvalue = shift;
	return unless $newvalue =~ /\d+/;
	$self->{'StartMonth'} = $newvalue;
}

sub StartDay {
	my $self = shift;
	return $self->{'StartDay'} unless (@_);
	my $newvalue = shift;
	return unless $newvalue =~ /\d+/;
	$self->{'StartDay'} = $newvalue;
}

sub StartHour {
	my $self = shift;
	return $self->{'StartHour'} unless (@_);
	my $newvalue = shift;
	return unless $newvalue =~ /\d+/;
	$self->{'StartHour'} = $newvalue;
}

sub StartMins {
	my $self = shift;
	return $self->{'StartMins'} unless (@_);
	my $newvalue = shift;
	return unless $newvalue =~ /\d+/;
	$self->{'StartMins'} = $newvalue;
}

sub EndYear {
	my $self = shift;
	return $self->{'EndYear'} unless (@_);
	my $newvalue = shift;
	return unless $newvalue =~ /\d+/;
	$self->{'EndYear'} = $newvalue;
}

sub EndMonth {
	my $self = shift;
	return $self->{'EndMonth'} unless (@_);
	my $newvalue = shift;
	return unless $newvalue =~ /\d+/;
	$self->{'EndMonth'} = $newvalue;
}

sub EndDay {
	my $self = shift;
	return $self->{'EndDay'} unless (@_);
	my $newvalue = shift;
	return unless $newvalue =~ /\d+/;
	$self->{'EndDay'} = $newvalue;
}

sub EndHour {
	my $self = shift;
	return $self->{'EndHour'} unless (@_);
	my $newvalue = shift;
	return unless $newvalue =~ /\d+/;
	$self->{'EndHour'} = $newvalue;
}

sub EndMins {
	my $self = shift;
	return $self->{'EndMins'} unless (@_);
	my $newvalue = shift;
	return unless $newvalue =~ /\d+/;
	$self->{'EndMins'} = $newvalue;
}

sub rawresult {
	my $self = shift;
	return @{$self->{'rawresult'}} unless (@_);
	my @recordset = @_;
	@{$self->{'rawresult'}} = @recordset;
}

sub headers {
	my $self = shift;
	my @records = $self->rawresult();
	my $csv = new Text::CSV_XS;
	if (!$csv->parse($records[0])) { 
		warn "Could not parse header"; 
		return; 
	}
	return $csv->fields();
}

sub count {
	my $self = shift;
	my @records = $self->rawresult();
	return scalar @records -1; # subtract one for the header
}

sub record {
	my ($self, $recnum) = @_;
	my @records = $self->rawresult();
	if ($recnum >= (scalar @records -1)) {
		warn "Invalid record requested ($recnum)";
		return;
	}
	my $csv = new Text::CSV_XS;
	if (!$csv->parse($records[$recnum+1])) { 
		warn "Could not parse record $recnum";
		return;
	}
#	%hash = map $self->headers(), $csv->fields();
	my %hash = ();
	my @headers = $self->headers();
	my @fields = $csv->fields();
	for (my $i = 0; $i <= $#headers; $i++) {
		$hash{$headers[$i]} = $fields[$i];
	}
	return \%hash;
}

=head1 NAME

CommSecure::Report.pm - Interface to CommSecure's web reporting system

=head1 SYNOPSIS

use CommSecure::Report;
$report = new CommSecure::Report(
                 -Username => "username",
                 -Password => "password");

=head1 DESCRIPTION

This library acts as an interface to the CommSecure web reporting system.
To use this library, you must have a username and password to the CommSecure
reporting interface. The default URL that it accesses is based from:
    https://clearance.commsecure.com.au/cgi-bin/Report.cgi

It requires the use of lynx-ssl to perform the request.

If the Username and Password are not specified when the object is created,
then they must be set manually and a call to fetch_data performed when
needed.

=head1 METHODS

 my $report = new CommSecure::Report(-Username => "username",
                                     -Password => "password");
 $report->Username
 $report->Password

 $report->fetch_data

 $report->count
 $report->record
 $report->headers
 
 $report->StartYear
 $report->StartMonth
 $report->StartDay
 $report->StartHour
 $report->StartMins
 $report->EndYear
 $report->EndMonth
 $report->EndDay
 $report->EndHour
 $report->EndMins

=head2 Username

This method either sets or gets the Username used for the web reporting.

=head2 Password

This method either sets or gets the Password used for the web reporting.

=head2 fetch_data

fetch_data does the work in actually retrieving the data from CommSecure's
reporting system. This is called when the object is constructed, if the
Username and Password are passed to the object. Otherwise, you must call
fetch_data manually after setting the username and password.

=head2 count

count returns the number of records returned by CommSecure

=head2 record($i)

record($i) returns the $i'th record from the retrieved data set. The
records are indexed from 0 to count-1. The returned record is a hash
consisting of field names and values.

=head2 headers

Calling headers returns an array of headers that is the first line returned
by CommSecure's reporting system. This isn't expected to be terribly useful -
listed purely for information purposes.
 
=head2 (Start|End)(Year|Month|Day|Hour|Mins)

Anything matching the above RE, is a variable that can be set to limit the
range of transactions retrieved. After setting these variables, you must
call fetch_data manually. You can also set these when the object is
constructed by passing them as parameters to new.

=head1 LICENCE

Report.pm - Perl CommSecure web reporting interface.
Copyright (C) 2002 Bernard Blackham <bernard@blackham.com.au>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

=head1 COPYRIGHT

Copyright (C) 2002 Bernard Blackham <bernard@blackham.com.au>

=cut 

1;
