package Discounts;
#
## This script is available under the Gnu General Public Licence (GPL)
##
## Written by James Bromberger (james@rcpt.to).
##
#
use strict;

sub new {
	my $proto = shift;
	my $class = ref($proto)||$proto;
	my $self = {};
	bless $self, $class;
	my %args = @_;
	$self->dbh($args{'-dbh'}) if defined $args{'-dbh'};
	$self->name($args{'-name'}) if defined $args{'-name'};
	$self->access_code($args{'-code'}) if defined $args{'-code'};
	$self->discount_percent($args{'-percent'}) if defined $args{'-percent'};
	$self->count($args{'-count'}) if defined $args{'-count'};

	if (defined $args{'-load'}) {
		$self->load(-id => $args{'-load'}) && return $self;
		return;
	}

	return $self;
}

sub dbh {
	my $self = shift;
	if (@_) {
		my $new_dbh = shift;
		$self->{'dbh'} = $new_dbh;
		}
	return $self->{'dbh'};
}


sub id {
	my $self = shift;
	if (@_) {
		my $new_id = shift;
		if ($new_id !~ /^\d+$/) {
			warn "Not an ID";
		}
		$self->{'id'} = $new_id;
	}
	return $self->{'id'};
}

sub name {
	my $self = shift;
	if (@_) {
		my $new_name = shift;
		return unless defined $new_name;
		return if length($new_name) < 2;
		return if length($new_name) > 30;
		$self->{'name'} = $new_name;
	}
	return $self->{'name'};
}

sub access_code {
	my $self = shift;
	if (@_) {
		my $code = shift;
		return unless defined $code ;
		$self->{'access_code'} = $code;
	}
	return $self->{'access_code'};
}

sub discount_percent {
	my $self = shift;
	if (@_) {
		my $discount = shift;
		if (not defined $discount) {
			warn "No discount";
			return;
		}
		return if $discount !~ /^\d+(\.\d+)?$/;;
		return if $discount > 100;
		$self->{'discount_percent'} = $discount;
	}
	return $self->{'discount_percent'};
}

sub count {
	my $self = shift;
	if (@_) {
		my $count = shift;
		return if $count!~ /^\d+(\.\d+)?$/;;
		return if $count > 100;
		$self->{'count'} = $count;
		}
	return $self->{'count'};
}


sub count_available {
	my $self = shift;
	my $sql = "SELECT count(1) as Count FROM Discounts_Used where Discount_ID = ?";
	my $sth = $self->dbh->prepare($sql);
	my $res = $sth->execute($self->id);
	if ($res != 1) {
		warn "Huh, not one result in count_available()?";
		return;
	}
	my $ref = $sth->fetchrow_hashref;
	my $count = $ref->{'Count'} || 0;
	$sth->finish;
	return $self->count - $count;
}


sub use {
	my $self = shift;
	my %args = @_;
	if (not defined $args{'-custnum'}) {
		warn "No custnum";
		return;
	}
	if (not defined $args{'-ip'}) {
		warn "No IP";
		return;
	}
	my $sth = $self->dbh->prepare("INSERT INTO Discounts_Used (Discount_ID, CustNum, Date, IP) VALUES (?, ?, now(), ?)");
	my $res = $sth->execute($self->id, $args{'-custnum'}, $args{'-ip'});
	$sth->finish;
	return $res;
}


sub load {
	my $self = shift;
	my %args = @_;
	my $sql = "SELECT * FROM Discounts WHERE ID = ?";
	if (not defined $args{'-id'} ) {
		warn "No constraint?";
		return;
	} elsif ($args{'-id'} !~ /^\d+$/) {
		warn "Not a DiscountID!";
		return;
	}
	#warn "SQL is $sql, " . $args{'-id'};
	my $sth = $self->dbh->prepare($sql);
	my $res = $sth->execute($args{'-id'});
	if ($res != 1) {
		warn "Found $res discounts";
		return;
		}
	my $ref = $sth->fetchrow_hashref;
	$self->id($ref->{'ID'});
	$self->discount_percent($ref->{'Discount_Percent'});
	$self->name($ref->{'Name'});
	$self->access_code($ref->{'Access_Code'});
	$self->count($ref->{'Count'});
	return 1;
}


sub find {
	my $self = shift;
	my %args = @_;
	my $dbh = $args{'-dbh'} || $self->dbh;
	my $sql = "SELECT ID FROM Discounts WHERE Access_Code = ?";
	my $sth = $dbh->prepare($sql);
	#warn "Doing $sql, " , $args{'-find'} ;
	my $res = $sth->execute($args{'-find'});
	if ($res != 1) {
		warn "No discount: " . $args{'-find'};
		return;
	}
	my $ref = $sth->fetchrow_hashref;
	my $d = $self->new(-dbh => $dbh, -load => $ref->{'ID'});
	$sth->finish;
	#warn "There are " . $d->count_available . " of these available.";
	return $d;
}


1;
