package Email;
#
## This script is available under the Gnu General Public Licence (GPL)
##
## Written by James Bromberger (james@rcpt.to).
##
#
use strict;

sub new {
	my $proto = shift;
	my $class = ref($proto)||$proto;
	my $self = {};
	bless $self, $class;
	my %args = @_;
	$self->dbh($args{'-dbh'}) if defined $args{'-dbh'};
	$self->to($args{'-to'}) if defined $args{'-to'};
	$self->cc($args{'-cc'}) if defined $args{'-cc'};
	$self->bcc($args{'-bcc'}) if defined $args{'-bcc'};
	$self->subject($args{'-subject'}) if defined $args{'-subject'};
	$self->content($args{'-content'}) if defined $args{'-content'};
	$self->sign($args{'-sign'}) if defined $args{'-sign'};

	$self->to && $self->subject && $self->content && warn "Sending message" && $self->_send && return 1;
	#warn "Returning object";
	return $self;
}

sub dbh {
	my $self = shift;
	if (@_) {
		my $new_dbh = shift;
		if (ref $new_dbh !~ "DBI::db") {
			warn "Not a DBI:db handle: $new_dbh";
			return;
		}
		$self->{'dbh'} = $new_dbh;
	}
	return $self->{'dbh'};
}

sub to {
	my $self = shift;
	if (@_) {
		my $new_to= shift;
		if ($new_to !~ /\@([\d\w-]+\.)+[\d\w-]+$/) {
			warn "Not an email address: $new_to";
			return;
		}
		$self->{'to'} = $new_to;
	}
	return $self->{'to'};
}

sub cc {
	my $self = shift;
	if (@_) {
		my $new_cc= shift;
		if ($new_cc !~ /\@([\d\w-]+\.)+[\d\w-]+$/) {
			warn "Not an email address: $new_cc";
			return;
		}
		$self->{'cc'} = $new_cc;
	}
	return $self->{'cc'};
}
sub bcc {
	my $self = shift;
	if (@_) {
		my $new_bcc= shift;
		if ($new_bcc !~ /\@([\d\w-]+\.)+[\d\w-]+$/) {
			warn "Not an email address: $new_bcc";
			return;
		}
		$self->{'bcc'} = $new_bcc;
	}
	return $self->{'bcc'};
}

sub content {
	my $self = shift;
	if (@_) {
		$self->{'content'} = shift;
		}
	return $self->{'content'};
}

sub subject {
	my $self = shift;
	if (@_) {
		$self->{'subject'} = shift;
		}
	return $self->{'subject'};
}

sub sign {
	my $self = shift;
	if (@_) {
		$self->{'sign'} = shift;
		}
	return $self->{'sign'};
}

sub _send {
	my $self = shift;
	if (not $self->to) {
		warn "No address to send to";
		return;
	} elsif (not $self->subject) {
		warn "No subject";
		return;
	} elsif (not $self->content) {
		warn "No content";
		return;
	}

	use Net::SMTP;
	my $mailer = Net::SMTP->new('localhost');
	if (not $mailer) {
		warn "No mailer: $!";
		return;
	}
	$mailer->mail("LCA-Organisers 2003 <lca-organisers\@conf.linux.org.au>");
	$mailer->to($self->to);
	$mailer->cc($self->cc) if defined($self->cc);
	$mailer->bcc($self->bcc) if defined($self->bcc);
	# When debugging, we can override all email to go to one person
	# by setting that address here, throwing away the supplied address.
	#$mailer->to("james\@rcpt.to");
	#warn "Sending mail to " . $self->to;
	#warn $self->content;
	$mailer->data;
	$mailer->datasend("From: LCA-Organisers 2003 <lca-organisers\@conf.linux.org.au>\n");
	$mailer->datasend("To: " . $self->to . "\n");
	$mailer->datasend("Subject: " . $self->subject . "\n");
	$mailer->datasend("Errors-To: postmaster\@conf.linux.org.au\n");
	$mailer->datasend("X-LCA-2003-Quote: Get some air! Linux.conf.au 2003.");
	$mailer->datasend("\n\n");
	if ($self->sign) {
		$mailer->datasend("----NEED TO CONFIGURE GPG SIGN----\n");
		$mailer->datasend($self->content);
		$mailer->datasend("----NEED TO CONFIGURE GPG SIGN----\n");
	} else {
		$mailer->datasend($self->content);
	}
	$mailer->dataend;
	$mailer->quit;
	return 1;
}


1;
