package Form;
#
## This script is available under the Gnu General Public Licence (GPL)
##
## Written by James Bromberger (james@rcpt.to).
##
#
use strict;
use lib qw(.);
use Variable;
use FormCookie;
use Registration_Type;
use Person;

# This library contains all the HTML code we need. It sould have the following
# methods that return code. It requires a DBH handle to be set on startup.
#
# 	* RegForm (with arguments for each field)
# 		=> returns HTML


sub new {
	my $proto = shift;
	my $class = ref($proto)||$proto;
	my $self = {};
	bless $self, $class;
	my %args = @_;
	$self->dbh($args{'-dbh'}) if defined $args{'-dbh'};
	$self->query($args{'-query'}) if defined $args{'-query'};
	$self->locked($args{'-locked'}) if defined $args{'-locked'};

	$self->{'times'} = $self->_get_all_times();
	return $self;
}

sub dbh {
	my $self = shift;
	if (@_) {
		my $new = shift;
		if (ref($new) !~ "DBI::db") {
			warn "Not a DBI:db handle: " . ref($new);
			return;
		}
		$self->{'dbh'} = $new;
	}
	return $self->{'dbh'};
}

sub query {
	my $self = shift;
	if (@_) {
		my $new_q = shift;
		if (ref($new_q) ne "CGI") {
			warn "Not a CGI";
			return;
		}
		$self->{'query'} = $new_q;
	}
	return $self->{'query'};
}

sub locked {
	my $self = shift;
	if (@_) {
		my $locked = shift;
		$self->{'locked'} = $locked?1:0;
	}
	return $self->{'locked'} || 0;
}

##### Time fun.

sub _format_period {
	my $self = shift;
	my $period = shift;
	my $s = $period % 60;
	$period = int($period / 60);
	my $m = $period % 60;
	$period = int($period / 60);
	my $h = $period % 24;
	$period = int($period / 24);
	my $d = $period % 7;
	$period = int($period / 7);
	my $w = $period % 52;
	$period = int($period / 52);
	return ($s, $m, $h, $d, $w, $period);
}

sub _human_period {
	my $self = shift;
	my $time = shift;
	my ($s, $m, $h, $d, $w, $y) = $self->_format_period($time);
	my $time_to_go;
	$time_to_go.= "$y years, " if $y;
	$time_to_go.= "$w weeks, " if $w;
	$time_to_go.= "$d days, " if $d;
	$time_to_go.= sprintf "%02d:%02d:%02d", $h, $m, $s;
	return $time_to_go;
	}

sub _item_times_string {
	my $self = shift;
	my $item = shift;
	if (ref($item) eq "ARRAY") {
		my $s = $self->_human_period($item->[0]);
		my $e = $self->_human_period($item->[1]);
		return "starts in $s, ends in $s";
		}
	else {
		my $t = "starts in " . $self->_human_period($item);
		}
	}

sub _get_time {
	my $self = shift;

	if (not defined $self->dbh) {
		warn "No dbh!";
		return;
	}
	my ($var, $name) = @_;
	my $sth = $self->dbh->prepare("SELECT UNIX_TIMESTAMP(At) as Time from Time_Periods where Name like ?");
	my $res = $sth->execute($name);
	if ($res == 1) {
		my $ref = $sth->fetchrow_hashref;
		#warn "$name: \$var = " . $$var . ", new is " . $ref->{'Time'};
		$$var = $ref->{'Time'};
	}
	$sth->finish;
}

sub _get_all_times {
	my $self = shift;
	my $times;
	$self->_get_time(\$times->{'conference'}->[0], "Conference Start");
	$self->_get_time(\$times->{'conference'}->[1], "Conference End");
	$self->_get_time(\$times->{'registration'}->[0], "Start Registrations");
	$self->_get_time(\$times->{'registration'}->[1], "Stop Registrations");
	return $times;
}






sub current_registrations {
	my $self = shift;
	return Variable->new(-dbh => $self->dbh, -name => "Num_Registered_And_Paid") || 0;
	my $current_registrants = 0;
	my $sth = $self->dbh->prepare("SELECT count(*) as Count from Person");
	my $res = $sth->execute;
	my $ref = $sth->fetchrow_hashref;
	$sth->finish;
	return $ref->{'Count'};
}




sub html_price_list {
	my $self = shift;
	my $html = "<h2>Prices</H2>\n";
	$html.= "<table border=0>\n";

	my @registration_types = Registration_Type->all(-dbh => $self->dbh);
	foreach (sort {$a->cost <=> $b->cost} @registration_types) {
		$html.= sprintf "<tr><td>%s</td><td>AU\$%s</td></tr>\n",
			$_->name, $_->cost;
	}
	$html.= "</table>";
	return $html;
}



sub populate_person {
	my $self = shift;

	my $person = Person->new(-dbh => $self->dbh);
	$person->name($self->query->param('name')||"");
	$person->company_name($self->query->param('company_name')||"");
	$person->address($self->query->param('address')||"");
	$person->postcode($self->query->param('postcode')||"");
	$person->state($self->query->param('state')||"");
	$person->country($self->query->param('country')||"");
	$person->email_address($self->query->param('email_address')||"");
	$person->telephone($self->query->param('telephone')||undef);
	$person->shirt_size($self->query->param('shirt_size')||"");
	$person->formal_dinner($self->query->param('dinner')||"");
	$person->vegetarian($self->query->param('vegetarian')||"");
	$person->registration_type($self->query->param('registration_type_id')) if defined $self->query->param('registration_type_id');
	#warn "Experience should be set here = " . $self->query->param('experience');
	$person->experience($self->query->param('experience')) if defined $self->query->param('experience');
	$person->preferred_shell($self->query->param('preferred_shell_id')) if defined $self->query->param('preferred_shell_id');
	$person->preferred_editor($self->query->param('preferred_editor_id')) if defined $self->query->param('preferred_editor_id');
	$person->preferred_distribution($self->query->param('preferred_distribution_id')) if defined $self->query->param('preferred_distribution_id');
	$person->accommodation_start($self->query->param('accommodation_start')||"");
	$person->accommodation_end($self->query->param('accommodation_end')||"");
	$person->special_requirements($self->query->param('special_requirements')||"");

	$person->attend_lca2002($self->query->param('lca2002'));
	$person->attend_lca2001($self->query->param('lca2001'));
	$person->attend_lca1999($self->query->param('lca1999'));
	$person->class($self->query->param('class'));
	return $person;
}

sub html {
	my $self = shift;
	my $person = shift;

	$person = Person->new(-dbh => $self->dbh) unless defined $person;

	my @chr_classes = qw (kernel-janitor newbie technical-writer developer hacker cracker script-kiddie packet-munger perl-monk python-jester C-pointer);
	my @chr_attrs = qw (drunk happy bold brave dashing bemused confused bewildered confident aloof tired weird overjoyed curious inquisitive daring);
	my $chr = $chr_attrs[rand(@chr_attrs)] . " " . $chr_classes[rand(@chr_classes)];
	$chr = ($chr =~ /^[aeiou]/?"an ":"a ") . $chr;
	$chr = "You are " . $chr;
	
	my $html;

	if ($self->locked) {
		$html.= "<h2>Check Details</h2>";
	} else {
		$html.= "<h2>Registration</h2>";
	}
  $html.=<<FOO;
  <p><strong>
<span style="color:red">*</span> may be shared with Currie Hall if you put 
in dates you want accommodation<br>
<span style="color:blue">#</span> may be shared with our caterers<br>
<span style="color: green">&Dagger;</span> may be printed in the Conference Handbook for professional registrants<br>
<span style="color:orange"">^</span> is a required field<br>
All fields may be used by Linux Australia
</strong>
</p>
FOO
  $html.=<<FOO;



<form method="post">

<table border=1 align="center" bgcolor="#CCCCCC" size="100%"><tr><td>

<table border=0 width=100% cellpadding=2 cellspacing=2>
FOO
  $html.= sprintf "<tr class='ROWSHADE1'><td align='right'>Class</td><td><input name='class' type='hidden' value='%s'>%s</td></tr>\n", 
  	($self->locked && $self->query->param('class'))?$self->query->param('class'):$chr, 
	($self->locked && $self->query->param('class'))?$self->query->param('class'):($chr . "<br><small>(reload now for a new class; this is just for fun)</small>");
  $html.= "<tr class='ROWSHADE2'>\n";


  $html.= sprintf "<td align='right'>%s%s%s<span style=\"color:red\">*</span><span style=\"color:blue\">#</span><span style=\"color: green\">&Dagger;</span><span style=\"color:orange\">^</span></td>", 
 	(($self->query->param('name') && ! $person->name) ? "<strong> ==&gt; ":""),
	"Full Name",
        (($self->query->param('name') && ! $person->name) ? "</strong>":"");
  $html.= sprintf "<td>%s</td></tr>\n", $person->ask_name_html('-locked' => $self->locked, -value => $person->name || $self->query->param('name') || "");


  $html.= "<tr class='ROWSHADE1'>\n";
  $html.= sprintf "<td align='right'>%s%s%s<span style=\"color:red\">*</span><span style=\"color:orange\">^</span></td>",
  	(($self->query->param('email_address') && ! $person->email_address) ? "<strong> ==&gt; ":""),
	"Email",
	(($self->query->param('email_address') && ! $person->email_address) ? "</strong>" : "");

  $html.= sprintf "<td>%s</td></tr>\n", $person->ask_email_html('-locked' => $self->locked, -value => $person->email_address ||  $self->query->param('email_address') || "");

  $html.= "<tr class='ROWSHADE2'>\n";
  $html.= "<td align='right'>Company Name <span style=\"color: green\">&Dagger;</span><br><small>(only for Professional registrants)</small></td>";
  $html.= sprintf "<td>%s</td></tr>\n", $person->ask_company_name('-locked' => $self->locked, -value => $person->company_name || $self->query->param('company_name') || '');

  $html.= "<tr class='ROWSHADE1'>\n";


  $html.= sprintf "<td align='right'>%s%s%s<span style=\"color:red\">*</span><br><small>(use int'l and area codes, eg +dd d dddd dddd)</small></td>",
    (($self->query->param('telephone') && ! $person->telephone) ? "<strong> ==&gt;" : ""),
    "Telephone",
    (($self->query->param('telephone') && ! $person->telephone) ? "</strong>":"");
  
  $html.= sprintf "<td>%s</td></tr>\n", $person->ask_telephone_html('-locked' => $self->locked, -value => $person->telephone || $self->query->param('telephone') || "");
  $html.= "<tr class='ROWSHADE2'>\n";

  $html.= sprintf "<td valign='top' align='right'>%s%s%s<span style=\"color:red\">*</span><span style=\"color:orange\">^</span><br>\n<small>(We may end up posting a conference CD)</small></td>",
    (($self->query->param('address') && ! $person->address) ? "<strong> ==&gt;":""),
    "Postal Address",
        (($self->query->param('address') && ! $person->address) ? "</strong>":"");
  
  $html.= sprintf "<td>%s<br>\n",$person->ask_address_html('-locked' => $self->locked, -value => $person->address || $self->query->param('address') || "");


  $html.= sprintf " %sState%s<span style=\"color:orange\">^</span>:&nbsp;%s %sPostcode%s<span style=\"color:orange\">^</span>:&nbsp;%s<br>\n", 
    (($self->query->param('state') && ! $person->state) ? "<strong>":""),
    (($self->query->param('state') && ! $person->state) ? "</strong>":""),
    $person->ask_state_html('-locked' => $self->locked, -value => $person->state||$self->query->param('state') || ""),
    (($self->query->param('postcode') && ! $person->postcode) ? "<strong>":""),
    (($self->query->param('postcode') && ! $person->postcode) ? "</strong>":""),
    $person->ask_postcode_html('-locked' => $self->locked, -value => $person->postcode || $self->query->param('postcode') || "");

  $html.= sprintf "Country <span style=\"color:orange\">^</span>: %s\n", $person->ask_country_html('-locked' =>  $self->locked, -value => $person->country);
  $html .= "<tr class=\"ROWSHADE1\">\n";
  $html.= sprintf "<td align=\"right\">Shirt Size</td><td>%s</td></tr>\n", $person->ask_shirt_html('-locked' =>  $self->locked, -value => $person->shirt_size);;

  $html.= "<tr class=\"ROWSHADE2\">\n";
  $html.= sprintf "<td align='right'>Registration Type</td><td>%s</td></tr>\n", $person->ask_registration_type_html('-locked' =>  $self->locked, -value => $person->registration_type);;
  $html.= "<tr class=\"ROWSHADE1\">\n";
#  $html.= sprintf "<td align=\"right\">Conference Dinner<br><small>(One dinner is included for professional registrants.<br>This is the Friday dinner only! eg. for parters to attend)</small></td><td>%s additional dinner(s), ", $person->ask_dinner_html('-locked' =>  $self->locked, -value => $person->formal_dinner);
  $html.= sprintf "<td align=\"right\">Conference Dinner<br><small>(One dinner is included for professional registrants.<br>This is the Friday dinner only! eg. for parters to attend)</small></td><td>Dinner(s) tickets can no longer be purchased, ";
  $html.= sprintf "%s Special diet <span style=\"color:red\">*</span><span style=\"color:blue\">#</span><br>\n", $person->ask_vegetarian_html('-locked' =>  $self->locked, -value => $person->vegetarian);
  $html.= "<small>Detail your special diet requirements below.</small> </td> </tr>";
  $html.= "<tr class=\"ROWSHADE2\">\n";
  $html.= sprintf "<td align=\"right\">Preferred Shell</td><td>%s</td></tr>\n",  $person->ask_preferred_html('-type' => "shell", '-locked' =>  $self->locked, -value => $person->preferred_shell);
  $html.= "<tr class=\"ROWSHADE1\">\n";
  $html.= sprintf "<td align=\"right\">Preferred Editor</td><td>%s</td></tr>\n", $person->ask_preferred_html('-type' => "editor", '-locked' =>  $self->locked, -value => $person->preferred_editor);
  $html.= "<tr class=\"ROWSHADE2\">\n";
  $html.= sprintf "<td align=\"right\">Preferred Distribution</td><td>%s</td></tr>\n", $person->ask_preferred_html('-type' => "distribution", '-locked' =>  $self->locked, -value => $person->preferred_distribution);
  $html.= "<tr class=\"ROWSHADE1\">\n";
  $html.= sprintf "<td align=\"right\">Your Experience Level</td><td>%s</td></tr>\n", $person->ask_experience_html('-locked' =>  $self->locked, -value => $person->experience);
  $html.= "<tr class=\"ROWSHADE2\">\n";
  $html.= sprintf "<td align=\"right\">%s Campus Accommodation Dates%s <span style=\"color:red\">*</span><br><small>dd-mmm, eg 20-Jan, or blank for none</small></td>",
  	(($self->query->param('accommodation_start') && ! $person->accommodation_start) || ($self->query->param('accommodation_end') && ! $person->accommodation_end) ? "<strong> ==&gt;":""),
  	(($self->query->param('accommodation_start') && ! $person->accommodation_start) || ($self->query->param('accommodation_end') && ! $person->accommodation_end) ? "</strong>":"");

  if ($self->locked) {
  	if ($person->accommodation_start && $person->accommodation_end) {
	  $html.= sprintf "<td>%s to %s <small>(billed &amp; organised separately)</small></td></tr>\n", $person->ask_accommodation_start_html('-locked' =>  $self->locked, -value => $person->accommodation_start || $self->query->param('accommodation_start') || ""), $person->ask_accommodation_end_html('-locked' =>  $self->locked, -value => $person->accommodation_end || $self->query->param('accommodation_end') || "") 
	  } else {
  		$html.= "<td>None requested.</td>\n";
	}
} else {
	$html.= sprintf "<td>%s to %s <small>(billed &amp; organised separately)</small></td></tr>\n", $person->ask_accommodation_start_html('-locked' =>  $self->locked, -value => $person->accommodation_start || $self->query->param('accommodation_start') || ""), $person->ask_accommodation_end_html('-locked' =>  $self->locked, -value => $person->accommodation_end || $self->query->param('accommodation_end') || "");
  }
  $html.= "<tr class=\"ROWSHADE1\">\n";
  $html.= sprintf "<td align=\"right\" valign=\"top\">Special Requirements <span style=\"color:red\">*</span><span style=\"color:blue\">#</span><br><small>Eg: food, disabled access, etc</small></td><td>%s</td></tr>\n", $person->ask_special_requirements_html('-locked' =>  $self->locked, -value => $person->special_requirements);

  $html.= "<tr class=\"ROWSHADE2\">\n";
  $html.= sprintf "<td align=\"right\" valign=\"top\">Did you attend LCA 2002 in Brisbane?</td><td>%s</td></tr>\n", $person->ask_attend(-locked => $self->locked, -value => $person->attend_lca2002, -when => "lca2002");
  $html.= "<tr class=\"ROWSHADE1\">\n";
  $html.= sprintf "<td align=\"right\" valign=\"top\">Did you attend LCA 2001 in Sydney?</td><td>%s</td></tr>\n", $person->ask_attend(-locked => $self->locked, -value => $person->attend_lca2001, -when => "lca2001");
  $html.= "<tr class=\"ROWSHADE2\">\n";
  $html.= sprintf "<td align=\"right\" valign=\"top\">Did you attend LCA 1999 (CALU) in Melbourne?</td><td>%s</td></tr>\n", $person->ask_attend(-locked => $self->locked, -value => $person->attend_lca1999, -when => "lca1999");

  if ($self->locked) {
	my $amount = $person->amount_total;
	#$amount+= $person->registration_type->cost;
	#if ($person->registration_type->name ne "Normal" && $person->formal_dinner) {
		#use ExtraCharges;
		#$amount+=ExtraCharges->new(-dbh => $self->dbh, -name => "Conference Dinner");
	#}
	# Was there a discount code?
	#warn "Look for discount in " . $person->special_requirements;
	use Discounts;
	my $discount;
	if (defined $person->special_requirements && $person->special_requirements =~ /\[([^\]]+)\]/) {
		$discount = Discounts->find(-dbh => $self->dbh, -find => $1);
		if (defined $discount) {
			$amount-= $amount * ($discount->discount_percent/100);
			$amount = sprintf "%.2f", $amount;
			warn "The discount code is $1: " . $discount->discount_percent;
		} else {
			warn "Tried to use discount $1, but couldnt find it";
		}
	}

	$html.= "<tr class=\"ROWSHADE2\">\n";
	$html.= sprintf "<td align=\"right\"><strong>AMOUNT YOU WILL BE CHARGED</strong></td><td><strong><big>AU\$ %s</big></strong> %s</td></tr>\n", $amount, (defined $discount?"(You got a " . $discount->discount_percent . "\% discount)":"");
  }
  $html.= "</table>\n\n";
  $html.= '<input type="hidden" name="display" value="c">' if $self->locked;
  $html.= "<p class='Note'>IFF this is correct, press confirm.<br> Otherwise scroll down further and edit.</p>" if $self->locked;
  $html.= '<p style="text-align:center">';

  if ($self->locked) {
	$html.= '<input style="background-color: yellow; font-weight: bold; font-size: large" type="submit" value="-- CONFIRM the above --">';
	$html.= '<p>If the above is wrong, edit below and press OK at the end of the page</p>';
  } else {
  	$html.= '<input type="submit" value="   OK   ">';
	$html.= '<input type="reset"  value=" cancel ">';
  }
  $html.= <<FOO;

</p>

</tr>
</td>
</table>
</form>
FOO
	return $html;
}

sub email_already_present {
	my $self = shift;

	if (defined $self->query->param('email_address') && $self->query->param('email_address') =~ /^[^\@]+\@([\w\d-]+\.)+[\w\d-]+$/) {
		# Does this address exist?
		my $sql = "select count(1) AS Num from Person where Email_Address = ?";
		my $sth = $self->dbh->prepare($sql);
		my $res = $sth->execute($self->query->param('email_address'));
		if ($res != 1) {
			warn "Err, weird: $res results";
			return 2;
		}
		my $ref = $sth->fetchrow_hashref;
		$res = $ref->{'Num'};
		if ($res == 0) {
			#warn "Not yet registered: $res";
			return;
		} else {
			warn "Already registered: $res times";
			return 1;
		}
	} else {
		warn "Invalid EMAIL: " . $self->query->param('email_address');
		return 3;
	}
};

sub is_valid {
	my $self = shift;

	if ($self->query->param('telephone') && $self->query->param('telephone') !~ /^\s*\+?[\d\s\-\(\)]+\s*$/) {
		warn "Invalid telephone: ". $self->query->param('telephone');
		return;
	}

	if (defined $self->query->param('address') && length($self->query->param('address')) < 5) {
		warn "Invalid address";
		return;
	}

	if (defined $self->query->param('postcode') && length($self->query->param('postcode')) < 2) {
		warn "Invalid Postcode";
		return;
	}

	if (not (defined $self->query->param('shirt_size') && $self->query->param('shirt_size') =~ /^x*[sml]$/i)) {
		warn "Invalid shirt size";
		return;
	}

	if (not (defined $self->query->param('registration_type_id') && $self->query->param('registration_type_id') =~ /^\d$/)) {
		warn "Invalid registration";
		return;
	}


	return 1;
}

sub html_header {
	my $self = shift;
	my %args = @_;
	my $html=<<FOO;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">
<html lang="EN">
<head>
FOO
	$html.= $args{'-header'} if $args{'-header'};
	$html.= "<link rel='stylesheet' type='text/css' href='style.css' />" if -r "style.css";
	$html.= "<link rel='stylesheet' type='text/css' href='../style.css' />" if -r "../style.css";
	$html.= "<link rel='stylesheet' type='text/css' href='../../style.css' />" if -r "../../style.css";
	$html.= "<link rel='stylesheet' type='text/css' href='../../../style.css' />" if -r "../../../style.css";


	$html.=<<FOO;

<meta name="description" content="Linux.conf.au | 2003 | Perth" />
<meta http-equiv="keywords" content="Linux Australia National Conference" />
<meta http-equiv="reply-to" content="lca-committee\@linux.org.au (LCA Committee)" />

<link rev="made" href="mailto:james\@rcpt.to" />
<link rel="contents" href="/" />
<link rel="chapter" href="/speakers.html" title="Speakers" />
<link rel="chapter" href="/venue.html" title="Venue" />
<link rel="chapter" href="/accommodation.html" title="Accommodation" />
<link rel="chapter" href="/cfp.html" title="Call For Papers" />
<link rel="chapter" href="/register.html" title="Registration" />
<link rel="chapter" href="/programme.html" title="Programme" />
<link rel="chapter" href="/contact.html" title="Contact" />

<link rel="search" href="http://www.google.com/" />


FOO
	if (defined $args{'-title'}) {
		$html.= sprintf "<title>%s</title>", $args{'-title'};
	} else {
		$html.= "<title>Linux.conf.au 2003 | Perth | Register</title>";
	}
	$html.=<<FOO;
</head>

<body>

<table border=0 width="100%">
<tr><td width=130 valign="top">
FOO

my $menu_file;
# Location in dev
$menu_file = "/home/james/public_html/conf/menu.html" if -r "/home/james/public_html/conf/menu.html";
# Location in test
$menu_file = "/home/james/public_html/2002/lca2003-temp/menu.html" if -r "/home/james/public_html/2002/lca2003-temp/menu.html";
# Location in prod
$menu_file = "/var/www/menu.html" if -r "/var/www/menu.html";
$menu_file = "menu.html" if -r "menu.html";
$menu_file = "../menu.html" if -r "../menu.html";
$menu_file = "../../menu.html" if -r "../../menu.html";
if (-r $menu_file) {
	if (open F, $menu_file) {
		while (<F>) {
			if ($_=~/src="([^"]+)"/ && $1!~/^http:/) {
				# Is this readable?
				if (-r $1) {
					# OK
				} elsif (-r "../$1") {
					$_=~s/src="([^"]+)"/src="..\/$1"/;
				} elsif (-r "../../$1") {
					$_=~s/src="([^"]+)"/src="..\/..\/$1"/;
				} elsif (-r "../../conf/$1") {
					$_=~s/src="([^"]+)"/src="..\/..\/conf\/$1"/;
				}
			}
			$html.= $_;
		}
		close F;
	} else {
		print "\n\n<p>Failed to open menu.</p>\n\n";
	}
} else {
	$html.= "<!-- $menu_file -->\n";
}

$html.=<<FOO;

</td><td valign="top">

FOO


	my $current_registrants = $self->current_registrations() || 0;
	if (defined $args{'-title'}) {
		$html.= sprintf "<h1 id=\"MyH1\">%s</h1>", $args{'-title'};
	} else {
		$html.= "<h1 id=\"MyH1\">Linux.conf.au 2003 | Register</h1>";
	}
#<h1 id="MyH1">Linux.conf.au 2003 | Register</h1>
	$html.=<<FOO;

<p class="Note">
Last night there were $current_registrants people registered!
FOO

	if (($self->{'times'}->{'conference'}->[0] - time()) > 0) {
		$html.= sprintf "LCA 2003 starts %s, in %s.\n",
			scalar localtime($self->{'times'}->{'conference'}->[0]),
			$self->_human_period(($self->{'times'}->{'conference'}->[0] - time()));
	} else {
		$html.= sprintf "LCA 2003 started at %s; it's now %s.\n",
			scalar localtime($self->{'times'}->{'conference'}->[0]),
       			scalar localtime();
	}
	$html.= "</p>\n";

	return $html;
}


sub html_footer {
	my $html=<<FOO;
<hr>
<address>
Scripting by James and Bernard.<br>
The original <strong>Tux</strong> penguin is copyright by Larry Ewing.<br>
<strong>Linux</strong> is a trademark of Linus Torvalds.<br>
&copy; 2002 <a href="http://www.linux.org.au/">Linux Australia</a>.
</address>
<!-- Scripting by James Bromberger -->
</td></tr>
</table>
</body>
</html>
FOO
	return $html;
}



sub already_registered_message {
	my $self = shift;

	my $sql = "SELECT *, Unix_timestamp(Date_Entered) as UTDE from Person where email_address like ?";
	my $sth = $self->dbh->prepare($sql);
	my $res = $sth->execute($self->query->param('email'));

	my $html = "<h2>Already Registered</h2>";
	while (my $ref = $sth->fetchrow_hashref) {
		$html.= sprintf "<p>Someone with the email address %s has already registered on %s WST from host %s. </p>",  $ref->{'Email_Address'}, scalar localtime($ref->{'UTDE'}), $ref->{'Host'};
	}

	$html.= "<p>If you would like your registration changed, please send the corrections to lca-organisers\@conf.linux.org.au. If you have registered and not yet paid, then you need to get your registrant ID (customer number) from us. We can email this to your address as a part of the details we have on you. If your email address has changed, then email us anyway and we'll sort it out</p>\n";
	$html.= "<p>Once you have your details, return to this web site and you can then pay! </p>\n";
	$html.= sprintf "<p><form action=\"my_details.cgi\" method=\"POST\"><input type=\"hidden\" name=\"email_address\" value=\"%s\">\n", $self->query->param('email_address');
	$html.= "<input type=\"submit\" value=\"Send your details to you via email\">\n";

	return $html;
}

1;
