package FormCookie;
#
## This script is available under the Gnu General Public Licence (GPL)
##
## Written by James Bromberger (james@rcpt.to).
##
#
sub new {
        my $proto = shift;
	my $class = ref($proto)||$proto;
	my $self = {};
	bless $self, $class;
	my %args = @_;
	$self->dbh($args{'-dbh'}) if defined $args{'-dbh'};
}

sub dbh {
	my $self = shift;
	if (@_) {
		$self->{'dbh'} = shift;
		}
	return $self->{'dbh'};
}

sub create {
	my $self = shift;
	my $tag = shift;
	# We tag some extra data with it we dont want out in userspace.
	# This tage is probably a ref to a hash of data values.
	my $expiry = time() + 3600; # seconds
	if (not $tag) {
		warn "No tag has been given";
		die;
	}
	use Storable qw(freeze thaw);
	my $frozen_tag = freeze $tag;

	# Need to get a unique cookie value by doing rand() and making 
	# sure we dont already have this value.
	my $sql = "SELECT * FROM Form_Cookie WHERE Cookie = ?";
	my $sth = $self->dbh->prepare($sql);
	my $cookie;
	my $res;
	do {
		$cookie = rand() * 10000;
		$res = $sth->execute($cookie);
	} until ($res == 0);
	$sth->finish;

	$sql = "INSERT INTO Form_Cookie (Cookie, Tag, Expiry) VALUES(?,?,?)";
	$sth = $self->dbh->prepare($sql);
	$res = $sth->execute($cookie, $frozen_tag, $expiry);
	$sth->finish;
	if ($res != 1) {
		die "Could not save cookie";
	}

	return $cookie;
}

sub find {
	my $self = shift;
	my $cookie = shift;
	my $sql = "SELECT * FROM Form_Cookie where Cookie = ?";
	my $sth = $self->dbh->prepare($sql);
	my $res = $sth->execute($cookie);
	if ($res != 1) {
		$sth->finish;
		warn "Found $res results looking for $cookie";
		return;
	}
	my $ref = $sth->fetchrow_hashref;
	use Storable qw(thaw);
	my $tag = thaw $ref->{'Tag'};
	$sth->finish;
	return $tag;
}

sub delete_old {
	my $self = shift;
	my $sql = "DELETE FROM Form_Cookie where Expiry < now()";
	my $sth = $self->dbh->prepare($sql);
	my $res = $sth->execute;
	$sth->finish;
}


1;
