package Misc_Payment;
#
## This script is available under the Gnu General Public Licence (GPL)
##
## Written by James Bromberger (james@rcpt.to).
##
#
use strict;

# This is to help with issuing a receipt to people who either dont pay 
# by credit card, or decide to purchase something after they get to 
# the conference. A payment has the following attributes
#
#   Amount - amount including cents as an integer
#   CustNum - which person in the Person table this is for
#   Descrption - to appear ont he bill against the line item
#   Note - any private notes for LCA admins; refernce notes, cheque nos...
#   Date - when the payment was made
#   Host - the host the payment submission was made from
#
# Obviously we don't want everyone updating this! 

sub new {
	my $proto = shift;
	my $class = ref($proto)||$proto;
	my $self = {};
	bless $self, $class;
	my %args = @_;
	$self->dbh($args{'-dbh'}) if defined $args{'-dbh'};
	$self->amount($args{'-amount'}) if defined $args{'-amount'};
	$self->custnum($args{'-custnum'}) if defined $args{'-custnum'};
	$self->host($args{'-host'}) if defined $args{'-host'};
	$self->customer_comments($args{'-customer_comments'}) if defined $args{'-customer_comments'};
	$self->private_comments($args{'-private_comments'}) if defined $args{'-private_comments'};
	$self->operator($args{'-operator'}) if defined $args{'-operator'};
	$self->load($args{'-id'}) if defined $args{'-id'};
	return $self;
}

sub dbh {
	my $self = shift;
	if (@_) {
		my $new_dbh = shift;
		if (ref $new_dbh !~ "DBI::db") {
			warn "Not a DBI:db handle: $new_dbh";
			return;
		}
		$self->{'dbh'} = $new_dbh;
	}
	return $self->{'dbh'};
}

sub id {
	my $self = shift;
	if (@_) {
		my $new_id = shift;
		if ($new_id !~ /^\d+$/) {
			warn "Not an ID";
			return;
		}
		$self->{'id'} = $new_id;
	}
	return $self->{'id'};
}

sub refnum {
	my $self = shift;
	return "mt" . $self->id;
}

sub date {
	my $self = shift;
	if (@_) {
		my $new_date = shift;
		if (not(defined $new_date) || $new_date !~ /^\d+$/) {
			defined $new_date && warn "Not a date: $new_date";
			defined $new_date || warn "Not a date: (undefined)";
			return;
		}
		$self->{'date'} = $new_date;
	}
	return $self->{'date'};
}

sub host {
	my $self = shift;
	if (@_) {
		my $new_host = shift;
		if ($new_host !~ /^([\w\d-]*\.)+[\w\d]+$/) {
			warn "Invalid host: $new_host";
			return;
		}
		$self->{'host'} = $new_host;
	}
	return $self->{'host'};
}

sub operator {
	my $self = shift;
	if (@_) {
		my $new_operator = shift;
		return unless defined $new_operator;
		$self->{'operator'} = $new_operator;
	}
	return $self->{'operator'};
}

sub amount {
	my $self = shift;
	if (@_) {
		my $new_amt = shift;
		if ($new_amt !~ /^\d+\.\d\d$/) {
			warn "Invalid amount: remember, no decimal: $new_amt";
			return;
		}
		$self->{'amount'} = $new_amt;
	}
	#warn "Amount is " . $self->{'amount'};
	return $self->{'amount'};
}

sub custnum {
	my $self = shift;
	if (@_) {
		$self->{'custnum'} = shift;
	}
	return $self->{'custnum'};
}

sub customer_comments{
	my $self = shift;
	if (@_) {
		my $new_desc = shift;
		if (not defined $new_desc) {
			warn "Invalid desc";
			return;
		}
		$self->{'customer_comments'} = $new_desc;
	}
	return $self->{'customer_comments'};
}

sub private_comments{
	my $self = shift;
	if (@_) {
		my $new_desc = shift;
		if (not defined $new_desc) {
			warn "Invalid desc";
			return;
		}
		$self->{'private_comments'} = $new_desc;
	}
	return $self->{'private_comments'};
}


sub load {
	my $self = shift;
	my $id = shift;
	if ($id !~ /^\d+$/) {
		warn "Not an ID!";
		return;
	}
	my $sql = "SELECT *, Unix_timestamp(Date) AS ctime_Date from Misc_Payments where ID = ?";
	my $sth = $self->dbh->prepare($sql);
	my $res = $sth->execute($id);
	if ($res != 1) {
		warn "Found $res entries";
		return;
		}
	my $ref = $sth->fetchrow_hashref;
	$self->id($ref->{'ID'});
	$self->date($ref->{'ctime_Date'});
	$self->host($ref->{'Host'});
	$self->amount($ref->{'Amount'});
	$self->customer_comments($ref->{'Customer_Comments'});
	$self->private_comments($ref->{'Private_Comments'}) if defined $self->{'Private_Comments'};
	$self->operator($self->{'Operator'}) if defined $self->{'Operator'};
	return 1;
}

sub is_valid {
	my $self = shift;
	return unless defined $self->dbh;
	return unless defined $self->amount;
	return unless defined $self->host;
	return unless defined $self->custnum;
	return 1;
}


sub as_html {
	my $self = shift;

	my $html;
	$html.= "<table border=1><caption>Misc Payment</caption>\n";
	$html.= sprintf "<tr><td align=\"right\">%s</td><td>%s</td></tr>\n", 
		"Operator", $self->operator if defined $self->operator;
	$html.= sprintf "<tr><td align=\"right\">%s</td><td>%s</td></tr>\n", 
		"Host", $self->host if defined $self->host;
	$html.= sprintf "<tr><td align=\"right\">%s</td><td>%s</td></tr>\n", 
		"Date", scalar(localtime($self->date)) . " WST +0800" if defined $self->date;
	$html.= sprintf "<tr><td align=\"right\">%s</td><td>%s</td></tr>\n", 
		"Amount (AUD\$)", "<strong>" . $self->amount . "</strong>" if defined $self->amount;
	$html.= sprintf "<tr><td align=\"right\">%s</td><td>%s</td></tr>\n", 
		"Customer Comments", $self->customer_comments if defined $self->customer_comments;
	$html.= sprintf "<tr><td align=\"right\">%s</td><td>%s</td></tr>\n", 
		"Private Comments", $self->private_comments if defined $self->private_comments;
	$html.= sprintf "<tr><td align=\"right\">%s</td><td><a href=\"person_show.cgi?custnum=\%s\">%s</a></tr>\n", 
		"CustNum", $self->custnum, $self->custnum if defined $self->custnum;
	my $person;
	($person, undef) = Person->find(-dbh => $self->dbh, -custnum => $self->custnum) if defined($self->custnum);
	$html.= sprintf "<tr><td align=\"right\">%s</td><td>%s</td></tr>\n", 
		"Name", $person->name if (defined($person) and defined($person->name));
	$html.= sprintf "<tr><td align=\"right\">%s</td><td>%s</td></tr>\n", 
		"Email", $person->email_address if defined $person->email_address;
	$html.= sprintf "<tr><td align=\"right\">%s</td><td>%s</td></tr>\n", 
		"Amount Owing (AUD\$)", $person->amount_owing if (defined($person) and defined ($person->amount_owing));
		

	$html.= "</table>\n";
	return $html;
}

sub save {
	my $self = shift;
	if (not defined $self->dbh) {
		warn "No DBH to save with";
		return;
	} elsif (not defined $self->custnum) {
		warn "No custnum defined";
		return;
	} elsif (not defined $self->date) {
		$self->date(time());
	} elsif (not defined $self->amount) {
		warn "No Amount defined";
		return;
	}

	my $sql = "INSERT INTO Misc_Payments (Date, Amount, CustNum, Host";
	$sql.= ", Customer_Comments" if $self->customer_comments;
	$sql.= ", Private_Comments " if $self->private_comments;
	$sql.= ", Operator " if $self->operator;
	$sql.= ") VALUES (FROM_UNIXTIME(?), ?, ?, ?";
	$sql.= ", " . $self->dbh->quote($self->customer_comments) if $self->customer_comments;
	$sql.= ", " . $self->dbh->quote($self->private_comments) if $self->private_comments;
	$sql.= ", " . $self->dbh->quote($self->operator) if $self->operator;
	$sql.= ")";

	#warn "SQL to save is $sql";

	my $sth = $self->dbh->prepare($sql);
	my $res = $sth->execute($self->date || time(), $self->amount, $self->custnum, $self->host);
	$sth->finish;
	warn "No result: $res" unless $res;
	$sql = "SELECT last_insert_id() as ID from Misc_Payments";
	$sth = $self->dbh->prepare($sql);
	$res = $sth->execute();
	my $ref = $sth->fetchrow_hashref;
	$self->id($ref->{'ID'});
	$sth->finish;
	return $res;
}

sub find {
	my $self = shift;
	my %args = @_;

	my $dbh = $args{'-dbh'};

	my $sql = "select ID from Misc_Payments";
	my $where;
	if(defined $args{'-amount'}) {
		if ($args{'-amount'} !~ /^\d+$/) {
			warn "Not an amount";
			return;
		}
		$where.= " AND " if $where;
		$where.= sprintf " Amount = %s", $dbh->quote($args{'-amount'});
	}

	if (defined $args{'-personid'}) {
		if ($args{'-amount'} !~ /^\d+$/) {
			warn "Not an ID";
			return;
		}
		$where.= " AND " if $where;
		$where.= sprintf " CustNum = %d", $args{'-custnum'};
	}
	$sql.= " WHERE " . $where if $where;
	my $sth = $args{'-dbh'}->prepare($sql);
	my $res = $sth->execute;
	if (not ($res > 0)) {
		warn "None found";
		return;
	}
	my @array;
	while (my $ref = $sth->fetchrow_hashref) {
		push @array, Misc_Payment->new('-dbh' => $args{'-dbh'}, -id => $ref->{'ID'});
	}
	return @array;
}

1;
