package Person;
#
## This script is available under the Gnu General Public Licence (GPL)
##
## Written by James Bromberger (james@rcpt.to).
##
#
use Preferred_Distributions;
use Preferred_Shells;
use Preferred_Editors;
use Registration_Type;
use Receipt;
use strict;

my $month_map = {
	'NOV' => '11', 'DEC' => '12', 'JAN' => '01', 'FEB' => '02',
	'MAR' => '03', 'APR' => '03', 'MAR' => '03', 'MAY' => '03',
	};

sub new {
	my $proto = shift;
	my $class = ref($proto)||$proto;
	my $self = {};
	bless $self, $class;
	my %args = @_;
	$self->dbh($args{'-dbh'}) if defined $args{'-dbh'};
	$self->name($args{'-name'}) if defined $args{'-name'};
	$self->email_address($args{'-email'}) if defined $args{'-email'};
	$self->telephone($args{'-telephone'}) if defined $args{'-telephone'};
	$self->vegetarian($args{'-vegetarian'}) if defined $args{'-vegetarian'};

	$self->shirt_size($args{'-shirt'}) if defined $args{'-shirt'};
	$self->registration_type($args{'-registration_type'}) if defined $args{'-registration_type'};
	$self->formal_dinner($args{'-dinner'}) if defined $args{'-dinner'};
	$self->preferred_shell($args{'-shell'}) if defined $args{'-shell'};
	$self->preferred_editor($args{'-editor'}) if defined $args{'-editor'};
	$self->preferred_distribution($args{'-distribution'}) if defined $args{'-distribution'};


	$self->address($args{'-address'}) if defined $args{'-address'};
	$self->country($args{'-country'}) if defined $args{'-country'};
	$self->state($args{'-state'}) if defined $args{'-state'};
	$self->postcode($args{'-postcode'}) if defined $args{'-postcode'};

	$self->paid($args{'-paid'}) if defined $args{'-paid'};

	$self->experience($args{'-experience'}) if defined $args{'-experience'};
	$self->special_requirements($args{'-requirements'}) if defined $args{'-requirements'};

	$self->remote_host($args{'-host'}) if defined $args{'-host'};

	if (defined $args{'-custnum'}) {
		$self->load(-custnum => $args{'-custnum'}) && return $self;
		return;
	}
	if (defined $args{'-load'}) {
		$self->load(-id => $args{'-load'}) && return $self;
		return;
	}

	return $self;
}

sub dbh {
	my $self = shift;
	if (@_) {
		my $new_dbh = shift;
		$self->{'dbh'} = $new_dbh;
		}
	return $self->{'dbh'};
}



sub country {
	my $self = shift;
	if (@_) {
		my $new_country = shift;
		return unless defined $new_country;
		return if length($new_country) > 100;
		return if length($new_country) < 3;
		if ($new_country =~ /^\s*(.*\S)\s*$/) {
			$self->{'country'} = $1;
		} else {
			return;
		}
	}
	return $self->{'country'};
}

sub ask_country_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not $args{'-value'}) {
			warn "No value!";
			return;
		}
		$html = sprintf "<input name=\"country\" type=\"hidden\" value=\"%s\">%s\n", $args{'-value'}, $args{'-display'} || $args{'-value'};
	} else {
		$html = sprintf "<input name=\"country\" size=25 value=\"%s\">\n", $args{'-value'}||"Australia";
	}

	return $html;
}

sub postcode {
	my $self = shift;
	if (@_) {
		my $new_pc = shift;
		return unless defined $new_pc;
		return if length($new_pc) > 15;
		return if length($new_pc) < 2;
		if ($new_pc =~ /^\s*(.*\S)\s*$/) {
			$self->{'postcode'} = $1;
		} else {
			return;
		}
	}
	return $self->{'postcode'};
}

sub company_name {
	my $self = shift;
	if (@_) {
		my $new_company = shift;
		return unless defined $new_company;
		if (length($new_company) > 255) {
			warn "Company name too long: " . length($new_company);
			return;
		}
		if ($new_company =~ /^\s*(.*\S)\s*$/) {
			$self->{'Company_Name'} = $1;
		} else {
			return;
		}
	}
	return $self->{'Company_Name'};
}

sub ask_company_name {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not $args{'-value'}) {
			#warn "No value for locked record";
			return;
		}
		$html.= sprintf "<input name=\"company_name\" type=\"hidden\" value=\"%s\">%s", $args{'-value'}, $args{'-value'};
	} else {
		$html = sprintf "<input name=\"company_name\" value=\"%s\" size=35>\n", $args{'-value'}||"";
	}
	return $html;
}

sub ask_postcode_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not $args{'-value'}) {
			warn "No value for locked record";
			return;
		}
		$html.= sprintf "<input name=\"postcode\" type=\"hidden\" value=\"%s\">%s", $args{'-value'}, $args{'-display'} || $args{'-value'};
	} else {
		$html = sprintf "<input name=\"postcode\" value=\"%s\" size=10>\n", $args{'-value'}||"";
	}

	return $html;
}

sub state {
	my $self = shift;
	if (@_) {
		my $new_state = shift;
		return unless defined $new_state;
		if ($new_state =~ /^\s*(.*\S)\s*$/) {
			return if length($1) > 50;
			return if length($1) < 2;
			$self->{'state'} = $1;
		} else {
			return;
		}

	}
	return $self->{'state'};
}

sub ask_state_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not $args{'-value'}) {
			warn "No value in locked form";
			return;
		}
		my $new_state = shift;
		$html = sprintf "<input name=\"state\" type=\"hidden\" value=\"%s\">%s\n", $args{'-value'}, $args{'-display'} || $args{'-value'};
	} else {
		$html = sprintf "<input name=\"state\" value=\"%s\" size=15>\n", $args{'-value'}||"";
	}

	return $html;
}

sub address {
	my $self = shift;
	if (@_) {
		my $new_street = shift;
		return unless defined $new_street;
		return if length($new_street) > 50;
		return if length($new_street) < 2;
		$self->{'address'} = $new_street;
	}
	return $self->{'address'};
}

sub ask_address_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not $args{'-value'}) {
			warn "No value in locked form";
			return;
		}
		$html = sprintf "<input name=\"address\" type=\"hidden\" value=\"%s\"><pre>%s</pre>", $args{'-value'}, $args{'-value'};
	} else {
		$html = sprintf "<textarea name=\"address\" rows=4 cols=30>%s</textarea>\n", $args{'-value'}||"";
	}
	return $html;
}

sub id {
	my $self = shift;
	if (@_) {
		my $new_id = shift;
		if ($new_id !~ /^\d+$/) {
			warn "Not an ID";
		}
		$self->{'id'} = $new_id;
	}
	return $self->{'id'};
}

sub name {
	my $self = shift;
	if (@_) {
		my $new_name = shift;
		return unless defined $new_name;
		if ($new_name =~ /^\s*(.*\S)\s*$/ ) {
			return if length($1) < 2;
			return if length($1) > 50;
			$self->{'name'} = $1;
		} else {
			return;
		}
	}
	return $self->{'name'};
}

sub ask_name_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not $args{'-value'}) {
			warn "No value in locked form";
			return;
		}
		my $new_name = shift;
		$html = sprintf "<input name=\"name\" type=\"hidden\" value=\"%s\">%s\n", $args{'-value'}, $args{'-value'};
	} else {
		$html = sprintf "<input name=\"name\" value=\"%s\" size=35>\n", $args{'-value'}||"";
	}
	return $html;
}

sub email_address {
	my $self = shift;
	if (@_) {
		my $new_email= shift;
		return unless defined $new_email;
		return if length($new_email) < 2;
		return if length($new_email) > 200;
		if ($new_email =~ /^\s*([^\s\@]+\@([\w\d-]+\.)+[\w\d]+)\s*$/) {
			#warn "Email address is set to $1";
			$self->{'email_address'} = $1;
		} else {
			return;
		}
	}
	return $self->{'email_address'};
}

sub ask_email_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not $args{'-value'}){
			warn "No value in locked form";
			return;
		}
		$html = sprintf "<input name=\"email_address\" type=\"hidden\" value=\"%s\">%s\n", $args{'-value'}, $args{'-value'};
	} else {
		$html = sprintf "<input name=\"email_address\" value=\"%s\" size=35>\n", $args{'-value'}||"";
	}
	return $html;
}

sub check_existing_email {
	my $self = shift;
	my $sql = "select * from Person where Email_Address like ?";
	my $sth = $self->dbh->prepare($sql);
	my $res = $sth->execute($self->email);
	$sth->finish;
	return $res;
}

sub class {
	my $self = shift;
	if (@_) {
		my $new_class = shift;
		return if not defined $new_class;
		return if length($new_class) > 50;
		return if length($new_class) < 2;
		$self->{'class'} = $new_class;
		}
	return $self->{'class'};
}


sub telephone {
	my $self = shift;
	if (@_) {
		my $new_phone = shift;
		return unless defined $new_phone;
		return if length($new_phone) < 2;
		return if length($new_phone) > 50;
		if ($new_phone =~ /^\s*(\+?[\d\s\-\(\)]+)\s*$/) {
			$self->{'telephone'} = $1;
		} else {
			return;
		}
	}
	return $self->{'telephone'};
}

sub ask_telephone_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not $args{'-value'}) {
			warn "No value in locked form";
			return;
		}
		$html = sprintf "<input name=\"telephone\" type=\"hidden\" value=\"%s\">%s\n", $args{'-value'}, $args{'-value'};
	} else {
		$html = sprintf "<input name=\"telephone\" value=\"%s\">\n", $args{'-value'}||"";
	}
	return $html;
}

sub formal_dinner {
	my $self = shift;
	if (@_) {
		my $new_dinner = shift;
		# Compatible with number or true/false
		if ($new_dinner =~ /^\d$/) {
			$self->{'formal_dinner'} = $new_dinner;
		} elsif ($new_dinner) {
			$self->{'formal_dinner'} = 1;
		} else {
			$self->{'formal_dinner'} = 0;
		}
	}
	return $self->{'formal_dinner'} || 0;  # Default: no
}

sub ask_dinner_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not exists $args{'-value'}) {
			warn "No value in locked form";
			return;
		}
		#warn "Value is " . $args{'-value'};
		$html = sprintf "<input type=\"hidden\" name=\"dinner\" value=\"%s\">%s\n", $args{'-value'}, $args{'-value'};
	} else {
		#$html = sprintf "<input type=\"checkbox\" name=\"dinner\" %s>\n", $args{'-value'}?"CHECKED":"";
		$html.= sprintf "<select name=\"dinner\">\n";
		for (my $i = 0; $i <= 4; $i++) {
			$html.= sprintf "<option value=\"$i\" %s>$i\n", (defined($args{'-value'}) && $args{'-value'} == $i) ? "SELECTED":"";
		}
		$html.= "</select>\n";
	}
	return $html;
}

sub vegetarian {
	my $self = shift;
	if (@_) {
		my $new_v = shift;
		$self->{'vegetarian'} = $new_v?1:0;
	}
	return $self->{'vegetarian'}||0;  # Default: no
}

sub ask_vegetarian_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not exists $args{'-value'}) {
			warn "No value in locked form";
			return;
		}
		$html = sprintf "<input name=\"vegetarian\" type=\"hidden\" %s>%s\n", $args{'-value'}, $args{'-value'}?"I have a":"No ";
	} else {
		$html = sprintf "<input type=\"checkbox\" name=\"vegetarian\" %s>\n", $args{'-value'}?"CHECKED":"";
	}
	return $html;
}

sub shirt_size {
	my $self = shift;
	if (@_) {
		my $new_shirt = uc(shift);
		return unless defined $new_shirt;
		return unless $new_shirt =~ /^S|M|L|X{1,3}L$/;
		$self->{'shirt_size'} = $new_shirt;
	}
	return $self->{'shirt_size'};
}

sub ask_shirt_html {
	my $self = shift;
	my %args = @_;
	my $html;
	my $new_shirt = "";
	if ($args{'-locked'}) {
		if (not $args{'-value'}) {
			warn "No value in locked form";
			return;
		}
		$html.= sprintf "<input name=\"shirt_size\" type=\"hidden\" value=\"%s\">%s", $args{'-value'}, $args{'-label'}||$args{'-value'};
	} else {
		$html.= "<select name=\"shirt_size\">\n";
		$html.= sprintf "<option value=\"S\" %s>Small\n", $args{'-value'} && $args{'-value'} eq "S"?"SELECTED":"";
		$html.= sprintf "<option value=\"M\" %s>Medium\n", $args{'-value'} && $args{'-value'} eq "M"?"SELECTED":"";
		$html.= sprintf "<option value=\"L\" %s>Large\n", ($args{'-value'} && $args{'-value'} eq "L"||not defined $args{'-value'})?"SELECTED":"";
		$html.= sprintf "<option value=\"XL\" %s>Extra Large\n", $args{'-value'} && $args{'-value'} eq "XL"?"SELECTED":"";
		$html.= sprintf "<option value=\"XXL\" %s>XX Large\n", $args{'-value'} && $args{'-value'} eq "XXL"?"SELECTED":"";
		$html.= sprintf "<option value=\"XXXL\" %s>Wow, real big.\n", $args{'-value'} && $args{'-value'} eq "XXXL"?"SELECTED":"";
		$html.= "</select>\n";
	}
	return $html;
}

sub registration_type {
	my $self = shift;
	if (@_) {
		my $new_type = shift;
		return unless $new_type;
		return unless $new_type =~ /^\d+$/;
		my $rt = Registration_Type->new(-dbh => $self->dbh, -id => $new_type);
		$self->{'registration_type'} = $rt;
	}
	return $self->{'registration_type'};
}

sub ask_registration_type_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not $args{'-value'}) {
			warn "No value in locked form";
			return;
		}
		if (not ref($args{'-value'})) {
			warn "Not a reg type";
			return;
		}
		$html.= sprintf "<input name=\"registration_type_id\" type=\"hidden\" value=\"%s\">%s", $args{'-value'}->id, $args{'-value'}->name;
	} else {
		my @registration_types = Registration_Type->all(-dbh => $self->dbh);
		$html.= "<select name=\"registration_type_id\">\n";
		TYPE: foreach (reverse sort {$a->cost <=> $b->cost} @registration_types) {
			next TYPE if ($_->name =~ /professional/i);
			$html.= sprintf "<option value=\"%s\" %s>%s\n", $_->id, (defined ($self->registration_type) && $_->id eq $self->registration_type->id?"SELECTED":""), $_->name;
		}
		$html.= "</select>";
	}
	return $html;

}

sub experience {
	my $self = shift;
	if (@_) {
		my $new_experience = shift;
		return unless defined $new_experience;
		if ($new_experience !~ /^\d$/) {
			warn "Bad experience: $new_experience";
			return;
		}
		#warn "Exp set to $new_experience";
		$self->{'experience'} = $new_experience;
	}
	return $self->{'experience'};
}

sub ask_experience_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not defined $args{'-value'}) {
			warn "No experience value in locked form from " . (caller(1))[3] . " from " . (caller(2))[3] , " val = " . $args{'-value'};
			return;
		}
		$html .= sprintf "<input name=\"experience\" type=\"hidden\" value=\"%s\">%s", $args{'-value'}, $args{'-value'} == 0 ?"Advanced": $args{'-value'} == 1 ? "Intermediate" : "Beginner";
	} else {
		$html.= "<select name=\"experience\">\n";
		$html.= sprintf "<option value=0 %s>Advanced\n", ($args{'-value'} && $args{'-value'} eq 0)?"SELECTED":"";
		$html.= sprintf "<option value=1 %s>Intermediate\n", ($args{'-value'} && $args{'-value'} eq 1)?"SELECTED":"";
		$html.= sprintf "<option value=2 %s>Beginner\n", ($args{'-value'} && $args{'-value'} eq 2) ? "SELECTED":"";
		$html.= "</select>";
	}
	return $html;
}

sub special_requirements {
	my $self = shift;
	if (@_) {
		my $new_reqs = shift;
		return unless defined $new_reqs;
		$self->{'requirements'} = $new_reqs;
		#warn "SR set to " . $self->{'requirements'};
	}
	return $self->{'requirements'};
}

sub ask_special_requirements_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not exists $args{'-value'}) {
			warn "No value in locked form";
			return;
		}
		$html.= sprintf "<input name=\"special_requirements\" type=\"hidden\" value=\"%s\">%s\n", $args{'-value'}||"", $args{'-value'} || "None";
	} else {
		$html.= sprintf "<textarea cols=30 rows=4 name=\"special_requirements\">%s</textarea>\n", $args{'-value'}||"";
	}
	return $html;
}

sub accommodation_start {
	my $self = shift;
	if (@_) {
		my $new_date= shift;
		return unless defined $new_date;
		return unless $new_date;
		#warn "Start Date set to $new_date from " . (caller(1))[3];
		if ($new_date =~ /^(\d{1,2})[\/-](\w+)$/) {
			my $month = $month_map->{uc($2)};
			$self->{'Accommodation_Start'}=sprintf "%4d-%02d-%02d", 
				$month>6?'2002':'2003',
				$month,
				$1;
		} elsif ($new_date =~ /^\d{4}-\d{2}-\d{2}$/) {
			$self->{'Accommodation_Start'}=$new_date;
		} else {
			warn "Date not OK: $new_date from " . (caller(1))[3];
		}
	}
	return $self->{'Accommodation_Start'};
}

#sub accommodation_start_mysql {
#	my $self = shift;
#	return unless defined $self->{'Accommodation_Start'};
#	if ($self->{'Accommodation_Start'} =~ /^(\d+)-(\w+)$/) {
#		#warn "1 = $1, 2 = $2, month_map(2) = " . $month_map->{uc($2)};
#		return $date;
#	}
#}


sub ask_accommodation_start_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not exists $args{'-value'}) {
			warn "No value in locked form";
			return;
		}
		$html.= sprintf "<input name=\"accommodation_start\" type=\"hidden\" value=\"%s\">%s", $args{'-value'}||"", $args{'-value'}||"None";
	} else {
		$html.= sprintf "<input name=\"accommodation_start\" value=\"%s\" size=7>", $args{'-value'}||"";
	}
	return $html;
}

sub ask_accommodation_end_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if ($args{'-locked'}) {
		if (not exists $args{'-value'}) {
			warn "No value in locked form";
			return;
		}
		$html.= sprintf "<input name=\"accommodation_end\" type=\"hidden\" value=\"%s\">%s", $args{'-value'}||"", $args{'-value'}||"None";
	} else {
		$html.= sprintf "<input name=\"accommodation_end\" value=\"%s\" size=7>", $args{'-value'} ||"";
	}
	return $html;
}

sub accommodation_end {
	my $self = shift;
	if (@_) {
		my $new_date= shift;
		return unless defined $new_date;
		return unless $new_date;
		if ($new_date =~ /^(\d+)[\/-](\w+)$/) {
			my $month = $month_map->{uc($2)};
			$self->{'Accommodation_End'} = sprintf "%4d-%02d-%02d", 
				$month>6?'2002':'2003',
				$month,
				$1;
		} elsif ($new_date =~ /^\d{4}-\d{2}-\d{2}$/) {
			$self->{'Accommodation_End'} = $new_date;
		} else {
			warn "Date not OK: $new_date from " . (caller(1))[3];
		}
	}
	return $self->{'Accommodation_End'};
}
#sub accommodation_end_mysql {
#	my $self = shift;
#	return unless defined $self->{'Accommodation_End'};
#	if ($self->{'Accommodation_End'} =~ /^(\d+)-(\w+)$/) {
#		return $date;
#	}
#}


sub attend_lca2002 {
	my $self = shift;
	if (@_) {
		my $new_answer = shift;
		return unless defined $new_answer;
		return unless $new_answer =~ /^\d$/;
		$self->{'lca2002'} = $new_answer;
	}
	return $self->{'lca2002'};
}
sub attend_lca2001 {
	my $self = shift;
	if (@_) {
		my $new_answer = shift;
		return unless defined $new_answer;
		return unless $new_answer =~ /^\d$/;
		$self->{'lca2001'} = $new_answer;
	}
	return $self->{'lca2001'};
}

sub attend_lca1999 {
	my $self = shift;
	if (@_) {
		my $new_answer = shift;
		return unless defined $new_answer;
		return unless $new_answer =~ /^\d$/;
		$self->{'lca1999'} = $new_answer;
	}
	return $self->{'lca1999'};
}


sub ask_attend {
	my $self = shift;
	my %args = @_;
	my $html;
	return unless defined $args{'-when'};
	if ($args{'-locked'}) {
		if (not defined $args{'-value'}) {
			#warn "No value in locked form";
			return;
		}
	$html.= sprintf "<input name=\"%s\" type=\"hidden\" value=\"%s\">%s", $args{'-when'}, $args{'-value'}, ($args{'-value'}?"Yes":"No");
	} else {
		$html.= sprintf "<input type=\"radio\" name=\"%s\" value=\"1\" %s> Yes <input type=\"radio\" name=\"%s\" value=\"0\" %s> No", 
			$args{'-when'}, 
			defined($args{'-value'}) && ($args{'-value'} eq "1" ? "CHECKED":"") || "",
			$args{'-when'},
			defined($args{'-value'}) && ($args{'-value'} eq "0" ? "CHECKED":"") || "";
	}
	return $html;
}


sub host {
	my $self = shift;
	if (@_) {
		my $new_host = shift;
		$self->{'host'} = $new_host;
	}
	return $self->{'host'};
}

sub date_entered {
	my $self = shift;
	if (@_) {
		my $new_date = shift;
		$self->{'date'} = $new_date;
	}
	return $self->{'date'};
}

sub amount_paid {
	my $self = shift;

	# The amount paid is the total of all receipts!
	$self->load_receipts unless defined $self->receipts;
	return 0 unless defined $self->receipts;
	my $amt = 0;
	foreach (@{$self->receipts}) {
		$amt+= $_->amount;
	}
	return sprintf("%.2f", $amt);
}

sub amount_total {
	my $self = shift;
	if (not defined $self->registration_type) {
		warn "No rego type";
		return;
	}
	use ExtraCharges;
	my $amount = 0;
	$amount+= $self->registration_type->cost;
	
	#warn "Person is after dinner" if $self->formal_dinner;
	my $dinner = ExtraCharges->new(-dbh => $self->dbh, -name => "Conference Dinner");

	# Handle professionals and multiple dinners
	if ($self->registration_type->name eq "Professional" && defined $self->custnum) {
		#warn "Taking off the price of one dinner";
		$amount += (($self->formal_dinner - 1) * $dinner);
	} else {
		$amount += ($self->formal_dinner * $dinner);
	}
	#$amount+= ExtraCharges->new(-dbh => $self->dbh, -name => "Conference Dinner") if (defined($self->formal_dinner) && ($self->formal_dinner) && ($self->registration_type->name ne "Professional"));

	# Was there a discount code?Discount words in square brackets. 
	# Only the first one.
	
	if (defined $self->special_requirements && $self->special_requirements =~ /^\[([^\]]+)\]/) {
		my $discount_word = $1;
		use Discounts;
		my $discount = Discounts->find(-dbh => $self->dbh, -find => $discount_word);
		$amount-= $amount * ($discount->discount_percent/100) if defined $discount;
	}


	return sprintf("%.2f", $amount);
}

sub amount_owing {
	my $self = shift;
	my $total = $self->amount_total;
	my $paid = $self->amount_paid;
	return unless defined($total);
	return unless defined($paid);
	return sprintf("%.2f", $total - $paid);
}

sub custnum {
	my $self = shift;
	if (@_) {
		my $new = shift;
		return unless defined $new;
		$self->{'custnum'} = $new;
	}
	return $self->{'custnum'};
}

sub _check_custnum_unique {
	my $self = shift;
	my $num = shift;
	my $sql = "Select count(1) as Count from Person where CustNum = ?";
	my $sth = $self->dbh->prepare($sql);
	my $res = $sth->execute($num);
	if ($res != 1) {
		warn "Err";
		return;
	}
	my $ref = $sth->fetchrow_hashref;
	my $c = $ref->{'Count'};
	$sth->finish;
	return $c;
}

sub _generate_custnum {
	my $self = shift;
	use Digest::MD5 qw(md5_base64);
	my $unique = 0;
	my $count = 0;

	while (not $unique) {
		$count++;
		my $attempt = md5_base64($self->email_address . $self->name . rand);
		# Remove plus, amp, and back/forward slash, space
		$attempt =~ s/[\+\&\\\/\s01lO]//g;
		$attempt = substr($attempt, 0, 16);
		my $res =  $self->_check_custnum_unique($attempt);
		if (not defined $res ) {
			warn "Something weird happened trying to determine uniqueness";
			return;
		} elsif ($res == 0) {
			# OK
			$unique = 1;
			$self->{'custnum'} = $attempt;
			return 1;
		}
		last if $count > 10;
	}
}

sub receipts {
	my $self = shift;
	if (@_) {
		map {push @{$self->{'receipts'}}, $_} @_;
		#warn sprintf "%s has %d receipts.\n", $self->name, ($#{$self->{'receipts'}}+1);
	}
	return $self->{'receipts'};
}

sub load_receipts {
	my $self = shift;
	return unless defined $self->dbh;
	return unless defined $self->custnum;
	$self->receipts(Receipt->find(-dbh => $self->dbh, -custnum => $self->custnum));
}

sub receipt_list {
	my $self = shift;
	$self->load_receipts unless defined $self->receipts;
	return "<p>No receipts found.</p>\n" unless defined $self->receipts;
	my $html;
	$html.= "<table>\n";
	$html.= $self->receipts->[0]->list_as_html_th;
	foreach (@{$self->receipts}) {
		$html.= $_->list_as_html;
	}
	$html.= "</table>\n";
	return $html;
}

sub preferred_shell {
	my $self = shift;
	if (@_) {
		my $new_shell_id = shift;
		if (not ($new_shell_id && $new_shell_id =~ /^\d+$/)) {
			warn "Not a shell ID: $new_shell_id";
			return;
		}
		my $s = Preferred_Shells->new(-dbh => $self->dbh, -id => $new_shell_id);
		if (not $s) {
			warn "Could not load shell!";
			return;
		}
		$self->{'preferred_shell'} = $s;
	}
	return $self->{'preferred_shell'};
}

sub ask_preferred_html {
	my $self = shift;
	my %args = @_;
	my $html;
	if (not defined $args{'-type'}) {
		warn "No type!";
		return;
	}
	if ($args{'-locked'}) {
		if (not $args{'-value'}) {
			warn "No value in locked form";
			return;
		}
		if (not ref $args{'-value'}) {
			warn "Not a ref";
			return;
		}
		$html.= sprintf "<input name=\"preferred_%s_id\" type=\"hidden\" value=\"%s\">%s\n", $args{'-type'}, $args{'-value'}->id,  $args{'-value'}->name;
	} else {
		my @collection;
		@collection = Preferred_Distributions->all(-dbh => $self->dbh) if $args{'-type'} eq "distribution";
		@collection = Preferred_Editors->all(-dbh => $self->dbh) if $args{'-type'} eq "editor";
		@collection = Preferred_Shells->all(-dbh => $self->dbh) if $args{'-type'} eq "shell";
		$html.= sprintf "<select name=\"preferred_%s_id\">\n", $args{'-type'};
		foreach (sort {$a->name cmp $b->name} @collection) {
			$html.= sprintf "<option value=\"%s\" %s>%s\n", $_->id, $args{'-value'} && $args{'-value'}->id eq $_->id?"SELECTED":"", $_->name;
		}
		$html.= "</select>\n";
	}
	return $html;
}

sub preferred_distribution {
	my $self = shift;
	if (@_) {
		my $new_distribution_id = shift;
		if (not ($new_distribution_id && $new_distribution_id =~ /^\d+$/)) {
			warn "Not a distributionID";
			return;
		}

		my $d = Preferred_Distributions->new(-dbh => $self->dbh, -id => $new_distribution_id);
		if (not $d) {
			warn "Could not load distribution!";
			return;
		}
		$self->{'preferred_distribution'} = $d;
	}
	return $self->{'preferred_distribution'};
}

sub preferred_editor {
	my $self = shift;
	if (@_) {
		my $new_ed_id = shift;
		if (not ($new_ed_id && $new_ed_id =~ /^\d+$/)) {
			warn "Not an editorID from " . (caller(1))[3];
			return;
		}

		my $e = Preferred_Editors->new(-dbh => $self->dbh, -id => $new_ed_id);
		if (not $e) {
			warn "Could not get editor";
			return;
		}
		$self->{'preferred_editor'} = $e;
	}
	return $self->{'preferred_editor'};
}

sub person_html {
	my $self = shift;
	my $html = "";
	$html.= "<table>\n";
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Name: ", ($self->name||"BLANK");
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Email: ", $self->email_address;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Telephone: ", $self->telephone;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Company: ", $self->company_name if defined $self->company_name;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Address: ", $self->address;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Postcode: ", $self->postcode if defined $self->postcode;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "State: ", $self->state if defined $self->state;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Country: ", $self->country if defined $self->country;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Shirt: ", $self->shirt_size;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Dinners: ", $self->formal_dinner;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Vegetarian: ", $self->vegetarian ? "Yes":"No";
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Experience: ", $self->experience;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Shell: ", $self->preferred_shell->name if defined $self->preferred_shell;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Editor: ", $self->preferred_editor->name if defined $self->preferred_editor;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Distribution: ", $self->preferred_distribution->name if defined $self->preferred_distribution;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Registration Type: ", $self->registration_type->name if defined $self->registration_type;
	$html.= sprintf "<tr><td>%s</td><td>%s</td></tr>\n", "Host IP: ", $self->host if defined $self->host;
	$html.= sprintf "<tr><td>%s</td><td>%s WST +0800</td></tr>\n", "Date Reg: ", scalar(localtime($self->date_entered)) if defined $self->date_entered;
	$html.= sprintf "<tr><td>Amount Owing: </td><td>%s</td></tr>\n", $self->amount_owing;
	
	$html.= sprintf "<tr><td>Attended LCA 2002: </td><td>%s</td></tr>\n", defined($self->attend_lca2002)?($self->attend_lca2002?"Yes":"No"):"-";
	$html.= sprintf "<tr><td>Attended LCA 2001: </td><td>%s</td></tr>\n", defined($self->attend_lca2001)?($self->attend_lca2001?"Yes":"No"):"-";
	$html.= sprintf "<tr><td>Attended LCA 1999: </td><td>%s</td></tr>\n", defined($self->attend_lca1999)?($self->attend_lca1999?"Yes":"No"):"-";
	$html.= "</table>";
	return $html;
}


sub person_text {
	my $self = shift;
	my $text;

	my $addr = $self->address||"";
	$addr =~ s/\n/\n\t\t\t/mg;
	my $special_req = $self->special_requirements || "";
	$special_req =~ s/\n/\n\t\t\t\t/gm;

	$text.= sprintf "               Name:\t%s\n", $self->name;
	$text.= sprintf "              Email:\t%s\n", $self->email_address;
	$text.= sprintf "          Telephone:\t%s\n", $self->telephone || "";
	$text.= sprintf "            Company:\t%s\n", $self->company_name if defined($self->company_name);
	$text.= sprintf "    Customer Number:\t%s (use this for paying)\n", $self->custnum;
	$text.= sprintf "            Address:\t%s\n", $addr;
	$text.= sprintf "              State:\t%s\n", $self->state;
	$text.= sprintf "           Postcode:\t%s\n", $self->postcode;
	$text.= sprintf "            Country:\t%s\n", $self->country;
	$text.= sprintf "         Shirt Size:\t%s\n", $self->shirt_size;
	$text.= sprintf "  Registration Type:\t%s\n", $self->registration_type->name;
	$text.= sprintf "  Conference Dinner:\t%s\n", $self->formal_dinner . ($self->registration_type->name eq "Professional"?" (1 was included as reg'd as Professional)":"");
	$text.= sprintf "Special Reqirements:\t%s\n", $special_req if defined $special_req;
	$text.= "\n\n";
	$text.= sprintf "   Fav Distribution:\t%s\n", $self->preferred_distribution->name;
	$text.= sprintf "    Preferred Shell:\t%s\n", $self->preferred_shell->name;
	$text.= sprintf "   Preferred Editor:\t%s\n", $self->preferred_editor->name;
	$text.= sprintf "  Attended LCA 2002:\t%s\n", defined($self->attend_lca2002)?($self->attend_lca2002?"Yes":"No"):"-";
	$text.= sprintf "  Attended LCA 2001:\t%s\n", defined($self->attend_lca2001)?($self->attend_lca2001?"Yes":"No"):"-";
	$text.= sprintf "  Attended LCA 1999:\t%s\n", defined($self->attend_lca1999)?($self->attend_lca1999?"Yes":"No"):"-";

	return $text;
}

sub person_register_message {
	my $self = shift;
	my $message = sprintf "Hello %s.\n\n", $self->name;

	$message.= <<FOO;

We recieved a registration request from someone today. Hopefully it came from 
you. The following details were entered. We've sent you this message straight 
away so you can be sure we have your information. Hopefully you have proceeded 
to the checkout and already paid for your registration for Linux.conf.au 2003. 
If not, please go to:

	https://www.linux.conf.au/register/checkout.cgi


..and enter your customer number below. If you lose this email, then you 
can get the details (including your customer number) sent to you again by 
returning to the Linux.conf.au 2003 website and entering your email address.

The details we have on record for you are:

FOO
	$message.= $self->person_text;
	$message.= "\n\n";
	if ($self->accommodation_start && $self->accommodation_end) {
		$message.= <<FOO;

Since you requested accommodation at Currie Hall, they have been emailed with 
your relevant details. You should grab the PDF form from the Linux.conf.au 
2003 web site, fill in your credit card details, and fax it to them.
	http://www.linux.conf.au/register/Ch-booking.pdf

There are only a set number of rooms already reserved, so getting this to them 
as soon as possible will avoid your disappointment and mean you won't be in 
a tent. The drop-bears are not a major hazard!
	http://members.ozemail.com.au/~kingpin1/dropbear.htm

FOO
	} else {
		$message.= <<FOO;
	

Since you did NOT request accommodation, you are being left to fend for 
yourself. The Yellow Pages (www.yellowpages.com.au) may help you to find 
hotels around the Crawley/Nedlands/City suburbs.
FOO

	}

	my $name = $self->name;
	my $date = scalar localtime;
	my $custnum = $self->custnum;
	my $reg_type = sprintf "(%-28s", $self->registration_type->name . " registration)";
	my $amount = $self->amount_owing;
	$message.= <<FOO;

If any of the details above are incorrect, please email the organisers at 
lca-organisers\@linux.conf.au with the content of this message, and a note 
detailing what you would like changed; we'll try and fix anything!

Well, that's about it for the moment. Be sure to check up on the Linux.conf.au 
web site for updates, and get on the lca-announce\@lists.linux.conf.au 
mailing list for our alerts!

We look forward to seeing you in Perth for Linux.conf.au 2003!



Regards,

    Linux.conf.au 2003 Registration Scripts
    on behalf of LCA2003 Organisers
    on behalf of Linux Australia, Inc.

-- 
 Linux.conf.au: The Australian Linux Techinical Conference
 The University of Western Australia, Perth, January 22 to January 25 2003
 Web: http://www.linux.conf.au/	  IRC Help: #lca2003 on irc.openprojects.net



------------------------------------------------------------------------------

                             Linux Australia Incorporated


			 	T A X    I N V O I C E



$date WST +0800

TAX INVOICE REFERENCE: $custnum



Attention: $name



INVOICE Linux.conf.au 2003 - The Australian Linux Technical Conference

This invoice has been issued as a result of an application to attend 
Linux.Conf.Au 2003, being held in Perth, Western Australia from 
Wednesday January 22nd until Saturday January 25th 2003.


ITEMS	                                         Currency        Amount
---------------------------------------------------------------------------
Linux Australia Membership			AUD		\$   0.00
Linux.conf.au $reg_type	AUD		\$ $amount
GST and Taxes					AUD		\$   0.00
==============							===========
Total                                           AUD		\$ $amount



This should be paid to Linux Australia, Incorporated, ABN 56 987 117 479. 

Payment by Australian personal cheque, international money order or bank 
draft may be sent to: Linux.Conf.Au 2003, c/o PLUG, PO Box 209, Bull 
Creek 6149, Western Australia, Australia. All cheques should be made 
payable to "Linux Australia, Inc.". Credit Card payments are available by 
visiting the confernece web site and quoting your customer number 
($custnum):

	http://www.linux.conf.au/register/checkout.cgi


If you are paying by cheque or money order, please print and attach this 
invoice. 


Note: The Australian Goods and Services Tax (GST) has NOT been collected 
(Linux Australia is not required to collect GST as it has a turnover under 
AUD \$100,000 per annum).


Enquiries may be emailed to the organisers: lca-organisers\@conf.linux.org.au.
Details about the conference may be found on the conference web site at 
http://www.linux.conf.au/ until the end of the conference, or at the Linux 
Australia web site at http://www.linux.org.au/.


FOO

	return $message;
}

sub person_csv{
	my $self = shift;
	my $text;
	my $addr = $self->address;
	$addr =~ s/[\n\r]/,/mg;

	$text.= sprintf "%s,%s,%s,\"%s\",%s,%s,%s, %s, %s, %s, %s, %s, %s, %s, %s\n", 
		$self->name, 
		$self->email_address,
		$self->telephone,
		$addr,
		$self->state,
		$self->postcode,
		$self->country,
		$self->shirt_size,
		$self->registration_type->name,
		$self->preferred_distribution->name,
		$self->preferred_shell->name,
		$self->preferred_editor->name;
		defined($self->attend_lca2002)?($self->attend_lca2002?"Yes":"No"):"",
		defined($self->attend_lca2001)?($self->attend_lca2001?"Yes":"No"):"",
		defined($self->attend_lca1999)?($self->attend_lca1999?"Yes":"No"):"",
	return $text;
}

sub xml {
	my $self = shift;
	my $xml = sprintf "<person name=\"%s\" email_address=\"%s\" custnum=\"%s\" ",
		$self->name, $self->email_address, $self->custnum;
	$xml.= sprintf "company_name=\"%s\" ", $self->company_name if defined $self->company_name;
	$xml.= sprintf "telephone=\"%s\" ", $self->telephone if defined $self->telephone;
	$xml.= sprintf "shirt_size=\"%s\" ", $self->shirt_size if defined $self->shirt_size;
	$xml.= sprintf "registration=\"%s\" ", $self->registration_type->name if defined($self->registration_type);
	$xml.= sprintf "dinner=\"%s\" ", $self->formal_dinner;
	$xml.= sprintf "shell=\"%s\" ", $self->preferred_shell->name if defined $self->preferred_shell;
	$xml.= sprintf "editor=\"%s\" ", $self->preferred_editor->name if defined $self->preferred_editor;
	$xml.= sprintf "distribution=\"%s\" ", $self->preferred_distribution->name if defined $self->preferred_distribution;
	$xml.= sprintf "address=\"%s\" ", $self->address if defined $self->address;
	$xml.= sprintf "postcode=\"%s\" ", $self->postcode if defined $self->postcode;
	$xml.= sprintf "state=\"%s\" ", $self->state if defined $self->state;
	$xml.= sprintf "country=\"%s\" ", $self->country if defined $self->country;
	$xml.= sprintf "special_requirements=\"%s\" ", $self->special_requirements if defined $self->special_requirements;
	$xml.= sprintf "experience=\"%s\" ", $self->experience if defined $self->experience;
	$xml.= sprintf "lca1999=\"%s\" ", $self->attend_lca1999 if defined $self->attend_lca1999;
	$xml.= sprintf "lca2001=\"%s\" ", $self->attend_lca2001 if defined $self->attend_lca1999;
	$xml.= sprintf "lca2002=\"%s\" ", $self->attend_lca2002 if defined $self->attend_lca1999;
	$xml.= sprintf "accommodation_start=\"%s\" accommodation_end=\"%s\" ", $self->accommodation_start, $self->accommodation_end if (defined($self->accommodation_start) && defined($self->accommodation_end));
	$xml.= sprintf "registered_date=\"%s\" ", $self->date_entered;
	$xml.= sprintf "registered_host=\"%s\" ", $self->host;
	$xml.= sprintf "class=\"%s\" ", $self->class if defined $self->class;
	$xml.= ">\n";
	$self->load_receipts unless defined $self->receipts;
	use CommSecure;
	my @t = CommSecure->find(-dbh=>$self->dbh, -custnum=>$self->custnum);
	if (@t) {
		$xml.= " <commsecure_list>\n";
		foreach (@t) {
			$xml.= sprintf "  <commsecure refnum=\"%s\">\n", $_->refnum;
			$xml.= sprintf "   <request date=\"%s\" host=\"%s\" merchant_num=\"%s\" description=\"%s\" />\n", $_->request->date, $_->request->host, ($_->request->merchantnum||""), $_->request->description;
			#if (defined $_->validate) {
			#	map {$xml.= sprintf "   <validate date=\"%s\" host=\"%s\"/>\n", $_->date, $_->host}, ${$_->validate};
			#}
			
			if (defined $_->result) {
				foreach (@{$_->result}) {
					$xml.= sprintf "   <result date=\"%s\" host=\"%s\" />\n", $_->date, $_->host;
				}
			}

			$xml.= "  </commsecure>\n";
		}
		$xml.= " </commsecure_list>\n";
	}
	if (defined $self->receipts) {
		$xml.= " <receipt_list>\n";
		foreach (@{$self->receipts}) {
			$xml.= sprintf "  <receipt date=\"%s\" refnum=\"%s\" amount=\"%s\" type=\"%s\" />\n", $_->date, $_->refnum, $_->amount, ($_->type||"");
		}
		$xml.= " </receipt_list>\n";
	}
	return $xml . "</person>\n";
}


sub data_ok {
	my $self = shift;
	if (!$self->name) {
		warn "Bad last_name";
		return;
	} elsif (!$self->email_address) {
		warn "Bad email";
		return;
	} elsif (!$self->shirt_size) {
		warn "Bad shirt";
		return;
	} elsif (!$self->telephone) {
		warn "Bad telephone";
		return;
	} elsif (!$self->address) {
		warn "Bad address";
		return;
	} elsif (!$self->postcode) {
		warn "Bad Postcode";
		return;
	} elsif (!$self->state) {
		warn "Bad state";
		return;
	} elsif (!$self->country) {
		warn "Bad country";
		return;
	} elsif (!$self->registration_type) {
		warn "Bad rego type";
		return;
	}
	return 1;
}

sub save {
	my $self = shift;
	# Correct the Professional Offset by adding one.to dinner
	if ($self->registration_type->name eq "Professional") {
		my $old_dinner = $self->formal_dinner;
		$self->formal_dinner($old_dinner + 1);
	}

	#warn("Registration is " . $self->registration_type->name . " which is ID " . $self->registration_type->id);
	my $sth = $self->dbh->prepare("INSERT INTO Person (Name, Company_Name, Email_Address, Telephone, Vegitarian, Shirt_Size, Registration_Type_ID, Formal_Dinner, Preferred_Shell_ID, Preferred_Editor_ID, Preferred_Distribution_ID, Address, State, PostCode, Country, Experience, Host, Date_Entered, CustNum, lca2002, lca2001, lca1999, Special_Requirements, Accomodation_Start, Accomodation_End, Class) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, now(), ?, ?, ?, ?, ?, ?, ?, ?)");
	$self->_generate_custnum unless defined $self->custnum;
		
	my $res = $sth->execute(
		$self->name,
		$self->company_name,
		$self->email_address,
		$self->telephone,
		$self->vegetarian,
		$self->shirt_size,
		$self->registration_type->id, 
		$self->formal_dinner,
		$self->preferred_shell->id,
		$self->preferred_editor->id,
		$self->preferred_distribution->id,
		$self->address,
		$self->state,
		$self->postcode,
		$self->country,
		$self->experience,
		$self->host,
		$self->custnum,
		defined($self->attend_lca2002)?$self->attend_lca2002:"NULL",
		defined($self->attend_lca2001)?$self->attend_lca2001:"NULL",
		defined($self->attend_lca1999)?$self->attend_lca1999:"NULL",
		$self->special_requirements,
		$self->accommodation_start|| undef,
		$self->accommodation_end|| undef,
		$self->class,
		);
	$sth->finish;
	if ($res == 1) {
		my $id_sql = "SELECT last_insert_id() as ID";
		$sth = $self->dbh->prepare($id_sql);
		my $res2 = $sth->execute;
		my $ref = $sth->fetchrow_hashref;
		$self->id($ref->{'ID'});
		$sth->finish;
	}
	# Did they use a Discount word - if so, then we need to tick off a 
	# discount from the list.
	if (defined $self->special_requirements && $self->special_requirements =~ /^\[([^\]]+)\]/) {
		my $discount_word = $1;
		use Discounts;
		my $discount = Discounts->find(-dbh => $self->dbh, -find => $discount_word);
		$discount->use(-custnum => $self->custnum, -ip => $self->host) if defined $discount;
		}
	return $res;

}


sub load {
	my $self = shift;
	my %args = @_;
	my $sql = "SELECT *, Unix_TIMESTAMP(Date_Entered) as Unix_Date_Entered from Person where ";
	my $param;
	if (defined $args{'-id'} ) {
		if ($args{'-id'} !~ /^\d+$/) {
			warn "Not a PersonID!";
			return;
		}
		$sql.= " ID ";
		$param = $args{'-id'};
	} elsif (defined $args{'-custnum'}) {
		$sql.= " CustNum ";
		$param = $args{'-custnum'};
	} else {
		warn "No constraint?";
		return;
	}
	$sql.= " = ?";
	#warn "SQL is $sql, $param";
	my $sth = $self->dbh->prepare($sql);
	my $res = $sth->execute($param);
	if ($res != 1) {
		warn "Found $res people";
		return;
	}
	my $ref = $sth->fetchrow_hashref;
	$self->id($ref->{'ID'});
	$self->name($ref->{'Name'});
	$self->company_name($ref->{'Company_Name'});
	$self->email_address($ref->{'Email_Address'});
	$self->telephone($ref->{'Telephone'});
	$self->shirt_size($ref->{'Shirt_Size'});
	$self->vegetarian($ref->{'Vegitarian'});
	$self->formal_dinner($ref->{'Formal_Dinner'});
	$self->registration_type($ref->{'Registration_Type_ID'});
	$self->preferred_shell($ref->{'Preferred_Shell_ID'});
	$self->preferred_editor($ref->{'Preferred_Editor_ID'});
	$self->preferred_distribution($ref->{'Preferred_Distribution_ID'});
	$self->date_entered($ref->{'Unix_Date_Entered'});
	$self->address($ref->{'Address'});
	$self->postcode($ref->{'PostCode'});
	$self->state($ref->{'State'});
	$self->country($ref->{'Country'});
	$self->experience($ref->{'Experience'});
	$self->special_requirements($ref->{'Special_Requirements'});
	$self->accommodation_start($ref->{'Accomodation_Start'});
	$self->accommodation_end($ref->{'Accomodation_End'});
	$self->host($ref->{'Host'});
	$self->custnum($ref->{'CustNum'});
	$self->class($ref->{'Class'});
	$self->attend_lca2002($ref->{'lca2002'});
	$self->attend_lca2001($ref->{'lca2001'});
	$self->attend_lca1999($ref->{'lca1999'});

	#$self->receipt($ref->{'Receipt'});
	#$self->receipt_emailed($ref->{'Receipt_Emailed'});
	return 1;
}


sub _strip_whitespace {
	my $a = shift;
	$a=~s/^\s+//g;
	$a=~s/\s+$//g;
	return $a;
}

sub find {
	my $self = shift;
	my %args = @_;
	my $dbh = $args{'-dbh'} || $self->dbh;

	my $sql = "SELECT ID from Person ";
	my @where;
	push @where, " Email_Address like '%" . _strip_whitespace($args{'-email'}) ."%' " if ($args{'-email'});
	push @where, " Name like '%" . $args{'-name'} . "%' " if ($args{'-name'});
	push @where, " Registration_Type_ID = " . $args{'-reg-id'} . " " if ($args{'-reg-id'});
	push @where, " CustNum = '" .  $args{'-custnum'} . "' " if defined  $args{'-custnum'};

	my $w = " where " . join("and", @where);
	$sql.=$w if @where;

	my $sth = $dbh->prepare($sql) or die "Cannot prepare: $!";
	my $res;
	#warn "Person::find SQL is $sql";
	$res = $sth->execute() or (die "Could not execute: $! " . $dbh->errstr && return);
	my @people;
	while (my $ref = $sth->fetchrow_hashref) {
		push @people, Person->new('-load' => $ref->{'ID'}, -dbh => $dbh);
	}
	return @people;
}

1;
