package Registration_Type;
#
## This script is available under the Gnu General Public Licence (GPL)
##
## Written by James Bromberger (james@rcpt.to).
##
#
sub new {
	my $proto = shift;
	my $class = ref($proto)||$proto;
	my $self = {};
	bless $self, $class;
	my %args = @_;
	$self->dbh($args{'-dbh'}) if defined $args{'-dbh'};
	$self->load($args{'-id'}) if defined $args{'-id'};
	return $self;
}

sub dbh {
	my $self = shift;
	if (@_) {
		my $new_dbh = shift;
		if (ref($new_dbh) !~ "DBI::db") {
			warn "Not a DBI:db handle: $new_dbh";
			return;
		}
		$self->{'dbh'} = $new_dbh;
	}
	return $self->{'dbh'};
}

sub name {
	my $self = shift;
	if (@_) {
		my $new_name = shift;
		if (length($new_name) > 100) {
			warn "Name too long";
			return;
		}
		$self->{'name'} = $new_name;
	}
	return $self->{'name'};
}

sub id {
	my $self = shift;
	if (@_) {
		my $new_id = shift;
		if (not $new_id || $new_id !~ /^\d+$/) {
			warn "Not an ID";
			return;
		}
		$self->{'id'} = $new_id;
	}
	return $self->{'id'};
}

sub cost {
	my $self = shift;
	if (@_) {
		my $new_cost= shift;
		if (not $new_cost || $new_cost !~ /^\d+(\.\d+)?$/) {
			warn "Not a cost";
			return;
		}
		$self->{'cost'} = $new_cost;
	}
	return $self->{'cost'};
}


sub load {
	my $self = shift;
	my $id = shift;
	if ($id !~ /^\d+$/) {
		warn "Not an ID!";
		return;
	}
	my $sql = "select * from Registration_Type where ID = ?";
	my $sth = $self->dbh->prepare($sql);
	my $res = $sth->execute($id);
	if ($res != 1) {
		warn "Found $res entries";
		return;
		}
	my $ref = $sth->fetchrow_hashref;
	$self->name($ref->{'Name'}) && $self->id($ref->{'ID'}) && $self->cost($ref->{'Cost'}) && return 1;
	return;
}

sub all {
	my $self = shift;
	my %args = @_;

	my $sql = "select ID from Registration_Type";
	my $sth = $args{'-dbh'}->prepare($sql);
	my $res = $sth->execute;
	if (not ($res > 0)) {
		warn "None found";
		return;
	}
	my @array;
	while (my $ref = $sth->fetchrow_hashref) {
		push @array, Registration_Type->new('-dbh' => $args{'-dbh'}, -id => $ref->{'ID'});
	}
	return @array;
}

1;
