#!/usr/bin/perl -w
#
# This script is available under the Gnu General Public Licence (GPL)
#
# Written by James Bromberger (james@rcpt.to).
#
#
use strict;
use lib qw(lib);
use CGI;
use DBI;
use Person;
use Form;

# 1 - Show form
# 2 - Before response, process data
#      - is data valid
#          - N -> complain LOUDLY, show form, highlight errors
#      - is email address already listed
#          - Y -> complain LOUDLY, show error message
#      - show confirmation
# 3 - Before response, save data, redirect to end page to prevent multiple saves
#
#
# Depends: libstorable-perl, libnet-perl

my $q = new CGI;
my $dbh = DBI->connect("dbi:mysql:database=LCA2003-Reg") or die "Can't connect";
my $f = Form->new(-query => $q, -dbh => $dbh);

# First, parse the input.
if (defined($q->param) && (not defined($q->param('c'))) && $f->is_valid) {
	#warn "Got valid form";
	my $person = $f->populate_person;
	if ($f->email_already_present) {
		print $q->header;
		print $f->html_header();
		print $f->already_registered_message();
		print $f->html_footer;
	} elsif (defined $q->param('display') && $q->param('display') ne "d") {
		# Save
		$person->host($q->remote_host);
		$person->_generate_custnum;
		$person->save;

		use Accommodation;
		use Email;

		# Change this to currie hall!
		Email->new(-to => 'currie@linux.conf.au', -subject => '[LCA2003] Accommodation Request: ' . $person->name, -content => Accommodation->generate_text(-person => $person)) if $person->accommodation_start && $person->accommodation_end;
		Email->new(-to => $person->email_address, -bcc => 'registrations@linux.conf.au', -subject => '[LCA2003] Registration Request: ' . $person->name, -content => $person->person_register_message);
		
		#warn "Self URL is " . $q->url(-path_info => 1);
		print $q->redirect($q->url(-path_info => 1) . "?c=" . $person->custnum);
	} else {
		# Confirm
		print $q->header();
		print $f->html_header();
		$f->locked(1);
		print "<p>The following details were given; please CHECK.</p>";
		print $f->html($person);
		$f->locked(0);
		print $f->html($person);
		print $f->html_footer();
	}
} elsif (defined($q->param) && defined($q->param('c'))) {
	# We are done - get the 'c' param with the cust num, and load 
	# the person object. We then have to see if the customer owes us 
	# money - they may not need to if they are on a freebie. We check 
	# the Person->amount to find out how much their bill is for, and 
	# check the CommSecure payments and Misc Payments 
	
	print $q->header;
	print $f->html_header();
	my $p = Person->new(-dbh => $dbh, -custnum => $q->param('c'));

	if (defined $p) {
		printf "<h2>Thank you %s (%s)</h2>\n", $p->name, $p->email_address;
		print "<pre>\n";
		print $p->person_text;
		print "</pre>\n";
		print "<p>We have your details, and if you entered your email address correctly, then you should get confirmation of your details soon. ";
		print "Since you requested Accommodation, Currie Hall at UWA will be in contact with you soon to sort that out." if (defined($p->accommodation_start) && defined($p->accommodation_end));
		print "</p>";
	
		if ($p->amount_total> $p->amount_paid) {
			printf "<p>However you are only HALF REGISTERED! We haven't had your payment yet for AU\$%s. We really hope you will pay by credit card now: it is secured, and at no time do you give us (LCA2003) your credit card details. We have arranged for CommSecure, a credit card transaction broker, to handle the billing.</p>", ($p->amount_total - $p->amount_paid);
			printf "<p>If you need to pay at a later date, you will need your <strong>customer number</strong> to be able to complete the transaciton. This can be emailed to you at any time (using the <a href=\"my_details.cgi\">my details</a> page). Your customer number is '<strong>%s</strong>'. This is case sensitive.</p>\n", $p->custnum;
			printf "<p>If you would like to pay now by credit card (yes, please!), then ...</p>\n";
			printf "<form action=\"checkout.cgi\" method=\"post\"><input type=\"hidden\" name=\"c\" value=\"%s\"><input type=\"submit\" value=\"Click here to pay by credit card, super securely with CommSecure\"></form></p>\n", $q->param('c');
			printf "<p>You may pay later by credit card, but you will get an annoying email message every week reminding you that you have to pay; you have three weeks to pay, after which we remove your details from the registration database. If you want to organise alternate payment, then you should email lca-organisers\@conf.linux.org.au RIGHT NOW! Print this page and attach it to any payments you want to post to us (Treasurer, Linux.conf.au 2003, C/o Perth Linux Users Group, PO Box 209, Bullcreek 6149, Western Australia).</p>";
		} else {
			print "<p>It seems you don't owe us any money, which means you are now done with this registration form.</p>";
		}

		print "<p>If you have any changes to your registration details, please email lca-organisers\@conf.linux.org.au. We prefer GPG signed email, and recommend you have been signed by a few people (ie: web of trust).</p>";

		print "<p>At any time you can go to the <a href=\"my_details.cgi\">my details</a> page, put in your email address, and have your details that we have on you sent to you so you can check it is all correct.</p>";
	} else {
		print "<h2>Not found</h2>\n";
		printf "<p>Hello %s. We can't find your details -- the Customer Number is not found in our database. Sorry, no luck.</p>", $q->remote_host;
	}

	print $f->html_footer;
} else {
	# Show form
	my $person = $f->populate_person;
	print $q->header();
	print $f->html_header();
	print $f->html($person);
	print $f->html_footer();
}
